/*
 * STARSTypes.h - <see below>.
 *
 * Copyright (c) 2015, 2016 Zephyr Software LLC 
 *
 * This file is part of the STARS (STatic Analyzer for Reliability ans Security) infrastructure.
 * This file may be used and modified for non-commercial purposes as long as 
 * all copyright, permission, and nonwarranty notices are preserved.  
 * Redistribution is prohibited without prior written consent from Zephyr Software LLC 
 *
 * Please contact the authors for restrictions applying to commercial use.
 *
 * THIS SOURCE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: Clark L. Coleman
 * e-mail: clc@zephyr-software.com
 * URL   : http://www.zephyr-software.com/
 *
 */

#ifndef STARSTYPES_H
#define STARSTYPES_H 1

// STARSTypes.h
//
// This header defines the typedefs needed for obtaining program information
//  from a disassembler or other program database, e.g. IDA Pro, the UVa IRDB,
//  another disassembler, etc.
//

#include <memory>
#include <vector>

#if 0
#include <string>
#include <utility>
#include <list>
#include <map>
#include <set>
#endif

#include <cstddef>
#include <cstdint>

// Mimic the MAXSTR constant from IDA Pro.
#define STARS_MAXSTR 1024

enum IBType {
	ZST_RETURN = 0,
	ZST_SWITCHTABLE = 1,
	ZST_INDIRCALL = 2,
	ZST_COMPUTEDGOTO = 3,   // heuristic identification, otherwise it would be ZST_SWITCHTABLE
	ZST_CODEADDRESSTAKEN = 4,
	ZST_UNREACHABLEBLOCK = 5
};

// Security policies for the Zephyr Security Toolkit.
enum ZST_Policy {
	ZST_DISALLOW = 0,
	ZST_WHITELIST = 1,
	ZST_BLACKLIST = 2,
	ZST_ALLOWALL = 3
};

// What type of system call, for Zephyr Security Toolkit monitoring.
enum ZST_SysCallType {
	ZST_UNMONITORED_CALL,
	ZST_HIGHPRIVILEGE_CALL,
	ZST_FILE_CALL,
	ZST_NETWORK_CALL
};

#ifdef __EA64__
// We imitate IDA Pro typedefs exactly, rather than assuming that unsigned long long is the same as uint64_t, etc.
// Imitate the IDA Pro ea_t.
typedef unsigned long long STARS_ea_t;
// Imitate the IDA Pro asize_t.
typedef unsigned long long STARS_asize_t;
// Imitate the IDA Pro uval_t.
typedef unsigned long long STARS_uval_t;
// Imitate the IDA Pro sval_t.
typedef          int64_t STARS_sval_t;
// Imitate the IDA Pro adiff_t.
typedef          int64_t STARS_adiff_t;
#else
typedef unsigned int STARS_ea_t;
typedef unsigned int STARS_asize_t;
typedef unsigned int STARS_uval_t;
typedef          int32_t STARS_sval_t;
typedef          int32_t STARS_adiff_t;
#endif

typedef ptrdiff_t STARS_ssize_t;

typedef uint32_t STARS_flags_t; // flags for each instruction, imitating <pro.h> typedef

// Match the IDA Pro BADADDR macro.
#define STARS_BADADDR ((STARS_ea_t) (-1))

class STARS_op_t;
typedef std::shared_ptr<class STARS_op_t> STARSOpndTypePtr;

// Function start and end addresses (for function entry chunks only).
//  Tracked in STARS_Program_t because IDA Pro 5.1 seemed to have a memory overwriting
//  problem when iterating through all functions in the program. An existing
//  func_t *ChunkInfo data structure was getting overwritten by one of the 
//  function func_t data structures, causing changes of startEA and endEA among
//  other things. Also helps identify chunked functions.
struct SMP_bounds_t {
	STARS_ea_t startEA;
	STARS_ea_t endEA;
};

struct SwitchTableInfo {  // info about a switch table
	STARS_ea_t DefaultJumpAddr; // address of jump to default case; precedes indirect jump; STARS_BADADDR if no default
	int DefaultCaseBlockNum; // block number of default case, which is the target of the default jump; could also be in CaseBlockNums
	int IndirJumpBlockNum; // block number with the INDIR_JUMP at the end
	int IDomBlockNum; // Block that is the immediate dominator of all blocks in the switch statement
	int FollowNodeNum;  // block number for follow node, to which all cases flow after their "break:" statements
	std::vector<int> CaseBlockNums; // block numbers for beginnings of cases in the switch
	std::vector<std::vector<STARS_sval_t> > IndexValue; // i.e. -2 in "case -2:" ; multiple case values can map to one block #
};

// The following operand data type constants match the constants from the <ua.hpp> IDA Pro header file with
//  the prefix STARS_ added. Review these constants in <ua.hpp> with each IDA Pro release to
//  see if new data types are added.

// from here..
#define STARS_dt_byte         0       // 8 bit
#define STARS_dt_word         1       // 16 bit
#define STARS_dt_dword        2       // 32 bit
#define STARS_dt_float        3       // 4 byte
#define STARS_dt_double       4       // 8 byte
#define STARS_dt_tbyte        5       // variable size (ph.tbyte_size)
#define STARS_dt_packreal     6       // packed real format for mc68040
// ...to here the order should not be changed, see mc68000
#define STARS_dt_qword        7       // 64 bit
#define STARS_dt_byte16       8       // 128 bit
#define STARS_dt_code         9       // ptr to code (not used?)
#define STARS_dt_void         10      // none
#define STARS_dt_fword        11      // 48 bit
#define STARS_dt_bitfild      12      // bit field (mc680x0)
#define STARS_dt_string       13      // pointer to asciiz string
#define STARS_dt_unicode      14      // pointer to unicode string
#define STARS_dt_3byte        15      // 3-byte data
#define STARS_dt_ldbl         16      // long double (which may be different from tbyte)
#define STARS_dt_byte32       17      // 256 bit
#define STARS_dt_byte64       18      // 512 bit

// Part of our solution to the dense encoding of x86-64 is to record some instruction
//  prefix and auxprefix info in each operand, so we can continue to do data flow analysis
//  on operands long after the instructions containing the operands have gone stale.
//  IDA Pro has macros that operate on their global variable 'cmd' which is only current
//  while analyzing an inst, and is too big to pass around with all operands in Phi DEFs,
//  LiveIn and LiveOut sets, SSA numbering and operand comparisons, etc. So, we record
//  the insnpref byte (a.k.a. cmd.rex) into each operand in the specflag4 field, then we
//  record the following two bits of info into unused bits in specflag4.
// These first five constants imitate IDA Pro header <intel.hpp> constants.
const int STARS_REX_W = 8;               // 64-bit operand size
const int STARS_REX_R = 4;               // modrm reg field extension
const int STARS_REX_X = 2;               // sib index field extension
const int STARS_REX_B = 1;               // modrm r/m, sib base, or opcode reg fields extension
const int STARS_VEX_L = 0x80;            // 256-bit operation (YMM register)

const int STARS_VEXPR = 0x40;            // IDA Pro vexpr() was true on inst containing current op_t
const int STARS_VSIB = 0x20;            // IDA Pro is_vsib() was true on inst containing current op_t

// Constants to imitate "feature" flags for each instruction as found in IDA Pro header <idp.hpp>
#define STARS_CF_STOP 0x00001  // Instruction doesn't pass execution to the
						// next instruction
#define STARS_CF_CALL 0x00002  // CALL instruction (should make a procedure here)
#define STARS_CF_CHG1 0x00004  // The instruction modifies the first operand
#define STARS_CF_CHG2 0x00008  // The instruction modifies the second operand
#define STARS_CF_CHG3 0x00010  // The instruction modifies the third operand
#define STARS_CF_CHG4 0x00020  // The instruction modifies 4 operand
#define STARS_CF_CHG5 0x00040  // The instruction modifies 5 operand
#define STARS_CF_CHG6 0x00080  // The instruction modifies 6 operand
#define STARS_CF_USE1 0x00100  // The instruction uses value of the first operand
#define STARS_CF_USE2 0x00200  // The instruction uses value of the second operand
#define STARS_CF_USE3 0x00400  // The instruction uses value of the third operand
#define STARS_CF_USE4 0x00800  // The instruction uses value of the 4 operand
#define STARS_CF_USE5 0x01000  // The instruction uses value of the 5 operand
#define STARS_CF_USE6 0x02000  // The instruction uses value of the 6 operand
#define STARS_CF_JUMP 0x04000  // The instruction passes execution using indirect
						// jump or call (thus needs additional analysis)
#define STARS_CF_SHFT 0x08000  // Bit-shift instruction (shl,shr...)
#define STARS_CF_HLL  0x10000  // Instruction may be present in a high level
						// language function.

// The following enumeration exactly matches the IDA Pro Intel x86 register enumeration "RegNo"
//  found in <intel.hpp> with the prefix STARS_x86_ added to each value.
enum STARS_RegNo
{
	STARS_x86_R_none = -1,
	STARS_x86_R_ax = 0,
	STARS_x86_R_cx = 1,
	STARS_x86_R_dx = 2,
	STARS_x86_R_bx = 3,
	STARS_x86_R_sp = 4,
	STARS_x86_R_bp = 5,
	STARS_x86_R_si = 6,
	STARS_x86_R_di = 7,
	STARS_x86_R_r8 = 8,
	STARS_x86_R_r9 = 9,
	STARS_x86_R_r10 = 10,
	STARS_x86_R_r11 = 11,
	STARS_x86_R_r12 = 12,
	STARS_x86_R_r13 = 13,
	STARS_x86_R_r14 = 14,
	STARS_x86_R_r15 = 15,

	STARS_x86_R_al = 16,
	STARS_x86_R_cl = 17,
	STARS_x86_R_dl = 18,
	STARS_x86_R_bl = 19,
	STARS_x86_R_ah = 20,
	STARS_x86_R_ch = 21,
	STARS_x86_R_dh = 22,
	STARS_x86_R_bh = 23,

	STARS_x86_R_spl = 24,
	STARS_x86_R_bpl = 25,
	STARS_x86_R_sil = 26,
	STARS_x86_R_dil = 27,

	STARS_x86_R_ip = 28,

	STARS_x86_R_es = 29,    // 0
	STARS_x86_R_cs = 30,    // 1
	STARS_x86_R_ss = 31,    // 2
	STARS_x86_R_ds = 32,    // 3
	STARS_x86_R_fs = 33,
	STARS_x86_R_gs = 34,

	STARS_x86_R_cf = 35,    // main cc's
	STARS_x86_R_zf = 36,
	STARS_x86_R_sf = 37,
	STARS_x86_R_of = 38,

	STARS_x86_R_pf = 39,    // additional cc's
	STARS_x86_R_af = 40,
	STARS_x86_R_tf = 41,
	STARS_x86_R_if = 42,
	STARS_x86_R_df = 43,

	STARS_x86_R_efl = 44,   // eflags

	// the following registers will be used in the disassembly
	// starting from ida v5.7

	STARS_x86_R_st0 = 45,   // floating point registers (not used in disassembly)
	STARS_x86_R_st1 = 46,
	STARS_x86_R_st2 = 47,
	STARS_x86_R_st3 = 48,
	STARS_x86_R_st4 = 49,
	STARS_x86_R_st5 = 50,
	STARS_x86_R_st6 = 51,
	STARS_x86_R_st7 = 52,
	STARS_x86_R_fpctrl = 53,// fpu control register
	STARS_x86_R_fpstat = 54,// fpu status register
	STARS_x86_R_fptags = 55,// fpu tags register

	STARS_x86_R_mm0 = 56,   // mmx registers
	STARS_x86_R_mm1 = 57,
	STARS_x86_R_mm2 = 58,
	STARS_x86_R_mm3 = 59,
	STARS_x86_R_mm4 = 60,
	STARS_x86_R_mm5 = 61,
	STARS_x86_R_mm6 = 62,
	STARS_x86_R_mm7 = 63,

	STARS_x86_R_xmm0 = 64,  // xmm registers
	STARS_x86_R_xmm1 = 65,
	STARS_x86_R_xmm2 = 66,
	STARS_x86_R_xmm3 = 67,
	STARS_x86_R_xmm4 = 68,
	STARS_x86_R_xmm5 = 69,
	STARS_x86_R_xmm6 = 70,
	STARS_x86_R_xmm7 = 71,
	STARS_x86_R_xmm8 = 72,
	STARS_x86_R_xmm9 = 73,
	STARS_x86_R_xmm10 = 74,
	STARS_x86_R_xmm11 = 75,
	STARS_x86_R_xmm12 = 76,
	STARS_x86_R_xmm13 = 77,
	STARS_x86_R_xmm14 = 78,
	STARS_x86_R_xmm15 = 79,
	STARS_x86_R_mxcsr = 80,

	STARS_x86_R_ymm0 = 81, // AVX 256-bit registers
	STARS_x86_R_ymm1 = 82,
	STARS_x86_R_ymm2 = 83,
	STARS_x86_R_ymm3 = 84,
	STARS_x86_R_ymm4 = 85,
	STARS_x86_R_ymm5 = 86,
	STARS_x86_R_ymm6 = 87,
	STARS_x86_R_ymm7 = 88,
	STARS_x86_R_ymm8 = 89,
	STARS_x86_R_ymm9 = 90,
	STARS_x86_R_ymm10 = 91,
	STARS_x86_R_ymm11 = 92,
	STARS_x86_R_ymm12 = 93,
	STARS_x86_R_ymm13 = 94,
	STARS_x86_R_ymm14 = 95,
	STARS_x86_R_ymm15 = 96,

	STARS_x86_R_bnd0 = 97, // MPX registers
	STARS_x86_R_bnd1 = 98,
	STARS_x86_R_bnd2 = 99,
	STARS_x86_R_bnd3 = 100,

	STARS_x86_R_xmm16 = 101, // AVX-512 extended XMM registers
	STARS_x86_R_xmm17 = 102,
	STARS_x86_R_xmm18 = 103,
	STARS_x86_R_xmm19 = 104,
	STARS_x86_R_xmm20 = 105,
	STARS_x86_R_xmm21 = 106,
	STARS_x86_R_xmm22 = 107,
	STARS_x86_R_xmm23 = 108,
	STARS_x86_R_xmm24 = 109,
	STARS_x86_R_xmm25 = 110,
	STARS_x86_R_xmm26 = 111,
	STARS_x86_R_xmm27 = 112,
	STARS_x86_R_xmm28 = 113,
	STARS_x86_R_xmm29 = 114,
	STARS_x86_R_xmm30 = 115,
	STARS_x86_R_xmm31 = 116,

	STARS_x86_R_ymm16 = 117, // AVX-512 extended YMM registers
	STARS_x86_R_ymm17 = 118,
	STARS_x86_R_ymm18 = 119,
	STARS_x86_R_ymm19 = 120,
	STARS_x86_R_ymm20 = 121,
	STARS_x86_R_ymm21 = 122,
	STARS_x86_R_ymm22 = 123,
	STARS_x86_R_ymm23 = 124,
	STARS_x86_R_ymm24 = 125,
	STARS_x86_R_ymm25 = 126,
	STARS_x86_R_ymm26 = 127,
	STARS_x86_R_ymm27 = 128,
	STARS_x86_R_ymm28 = 129,
	STARS_x86_R_ymm29 = 130,
	STARS_x86_R_ymm30 = 131,
	STARS_x86_R_ymm31 = 132,

	STARS_x86_R_zmm0 = 133, // AVX-512 ZMM registers
	STARS_x86_R_zmm1 = 134,
	STARS_x86_R_zmm2 = 135,
	STARS_x86_R_zmm3 = 136,
	STARS_x86_R_zmm4 = 137,
	STARS_x86_R_zmm5 = 138,
	STARS_x86_R_zmm6 = 139,
	STARS_x86_R_zmm7 = 140,
	STARS_x86_R_zmm8 = 141,
	STARS_x86_R_zmm9 = 142,
	STARS_x86_R_zmm10 = 143,
	STARS_x86_R_zmm11 = 144,
	STARS_x86_R_zmm12 = 145,
	STARS_x86_R_zmm13 = 146,
	STARS_x86_R_zmm14 = 147,
	STARS_x86_R_zmm15 = 148,
	STARS_x86_R_zmm16 = 149,
	STARS_x86_R_zmm17 = 150,
	STARS_x86_R_zmm18 = 151,
	STARS_x86_R_zmm19 = 152,
	STARS_x86_R_zmm20 = 153,
	STARS_x86_R_zmm21 = 154,
	STARS_x86_R_zmm22 = 155,
	STARS_x86_R_zmm23 = 156,
	STARS_x86_R_zmm24 = 157,
	STARS_x86_R_zmm25 = 158,
	STARS_x86_R_zmm26 = 159,
	STARS_x86_R_zmm27 = 160,
	STARS_x86_R_zmm28 = 161,
	STARS_x86_R_zmm29 = 162,
	STARS_x86_R_zmm30 = 163,
	STARS_x86_R_zmm31 = 164,

	STARS_x86_R_k0 = 165, // AVX-512 opmask registers
	STARS_x86_R_k1 = 166,
	STARS_x86_R_k2 = 167,
	STARS_x86_R_k3 = 168,
	STARS_x86_R_k4 = 169,
	STARS_x86_R_k5 = 160,
	STARS_x86_R_k6 = 171,
	STARS_x86_R_k7 = 172,

	STARS_x86_R_last,
};

typedef STARS_RegNo STARS_regnum_t;

inline bool STARS_x86_is_segreg(int r) { return r >= STARS_x86_R_es && r <= STARS_x86_R_gs; };

// Maximum operands per instruction, imitating <ua.hpp>.
#define STARS_UA_MAXOP  6

// The following enumeration exactly matches the IDA Pro Intel x86 instruction enumeration
//  found in <allins.hpp>. The prefix NN_ has been changed to STARS_NN_ for each value. With
//  each release of IDA Pro, the <allins.hpp> file must be inspected to see if new opcodes
//  have been added by the out-of-control maniacs at Intel. If so, copy, paste, and replace
//  the NN_ prefix with STARS_NN_ here.

enum
{
	STARS_NN_null = 0,            // Unknown Operation
	STARS_NN_aaa,                 // ASCII Adjust after Addition
	STARS_NN_aad,                 // ASCII Adjust AX before Division
	STARS_NN_aam,                 // ASCII Adjust AX after Multiply
	STARS_NN_aas,                 // ASCII Adjust AL after Subtraction
	STARS_NN_adc,                 // Add with Carry
	STARS_NN_add,                 // Add
	STARS_NN_and,                 // Logical AND
	STARS_NN_arpl,                // Adjust RPL Field of Selector
	STARS_NN_bound,               // Check Array Index Against Bounds
	STARS_NN_bsf,                 // Bit Scan Forward
	STARS_NN_bsr,                 // Bit Scan Reverse
	STARS_NN_bt,                  // Bit Test
	STARS_NN_btc,                 // Bit Test and Complement
	STARS_NN_btr,                 // Bit Test and Reset
	STARS_NN_bts,                 // Bit Test and Set
	STARS_NN_call,                // Call Procedure
	STARS_NN_callfi,              // Indirect Call Far Procedure
	STARS_NN_callni,              // Indirect Call Near Procedure
	STARS_NN_cbw,                 // AL -> AX (with sign)
	STARS_NN_cwde,                // AX -> EAX (with sign)
	STARS_NN_cdqe,                // EAX -> RAX (with sign)
	STARS_NN_clc,                 // Clear Carry Flag
	STARS_NN_cld,                 // Clear Direction Flag
	STARS_NN_cli,                 // Clear Interrupt Flag
	STARS_NN_clts,                // Clear Task-Switched Flag in CR0
	STARS_NN_cmc,                 // Complement Carry Flag
	STARS_NN_cmp,                 // Compare Two Operands
	STARS_NN_cmps,                // Compare Strings
	STARS_NN_cwd,                 // AX -> DX:AX (with sign)
	STARS_NN_cdq,                 // EAX -> EDX:EAX (with sign)
	STARS_NN_cqo,                 // RAX -> RDX:RAX (with sign)
	STARS_NN_daa,                 // Decimal Adjust AL after Addition
	STARS_NN_das,                 // Decimal Adjust AL after Subtraction
	STARS_NN_dec,                 // Decrement by 1
	STARS_NN_div,                 // Unsigned Divide
	STARS_NN_enterw,              // Make Stack Frame for Procedure Parameters
	STARS_NN_enter,               // Make Stack Frame for Procedure Parameters
	STARS_NN_enterd,              // Make Stack Frame for Procedure Parameters
	STARS_NN_enterq,              // Make Stack Frame for Procedure Parameters
	STARS_NN_hlt,                 // Halt
	STARS_NN_idiv,                // Signed Divide
	STARS_NN_imul,                // Signed Multiply
	STARS_NN_in,                  // Input from Port
	STARS_NN_inc,                 // Increment by 1
	STARS_NN_ins,                 // Input Byte(s) from Port to String
	STARS_NN_int,                 // Call to Interrupt Procedure
	STARS_NN_into,                // Call to Interrupt Procedure if Overflow Flag = 1
	STARS_NN_int3,                // Trap to Debugger
	STARS_NN_iretw,               // Interrupt Return
	STARS_NN_iret,                // Interrupt Return
	STARS_NN_iretd,               // Interrupt Return (use32)
	STARS_NN_iretq,               // Interrupt Return (use64)
	STARS_NN_ja,                  // Jump if Above (CF=0 & ZF=0)
	STARS_NN_jae,                 // Jump if Above or Equal (CF=0)
	STARS_NN_jb,                  // Jump if Below (CF=1)
	STARS_NN_jbe,                 // Jump if Below or Equal (CF=1 | ZF=1)
	STARS_NN_jc,                  // Jump if Carry (CF=1)
	STARS_NN_jcxz,                // Jump if CX is 0
	STARS_NN_jecxz,               // Jump if ECX is 0
	STARS_NN_jrcxz,               // Jump if RCX is 0
	STARS_NN_je,                  // Jump if Equal (ZF=1)
	STARS_NN_jg,                  // Jump if Greater (ZF=0 & SF=OF)
	STARS_NN_jge,                 // Jump if Greater or Equal (SF=OF)
	STARS_NN_jl,                  // Jump if Less (SF!=OF)
	STARS_NN_jle,                 // Jump if Less or Equal (ZF=1 | SF!=OF)
	STARS_NN_jna,                 // Jump if Not Above (CF=1 | ZF=1)
	STARS_NN_jnae,                // Jump if Not Above or Equal (CF=1)
	STARS_NN_jnb,                 // Jump if Not Below (CF=0)
	STARS_NN_jnbe,                // Jump if Not Below or Equal (CF=0 & ZF=0)
	STARS_NN_jnc,                 // Jump if Not Carry (CF=0)
	STARS_NN_jne,                 // Jump if Not Equal (ZF=0)
	STARS_NN_jng,                 // Jump if Not Greater (ZF=1 | SF!=OF)
	STARS_NN_jnge,                // Jump if Not Greater or Equal (ZF=1)
	STARS_NN_jnl,                 // Jump if Not Less (SF=OF)
	STARS_NN_jnle,                // Jump if Not Less or Equal (ZF=0 & SF=OF)
	STARS_NN_jno,                 // Jump if Not Overflow (OF=0)
	STARS_NN_jnp,                 // Jump if Not Parity (PF=0)
	STARS_NN_jns,                 // Jump if Not Sign (SF=0)
	STARS_NN_jnz,                 // Jump if Not Zero (ZF=0)
	STARS_NN_jo,                  // Jump if Overflow (OF=1)
	STARS_NN_jp,                  // Jump if Parity (PF=1)
	STARS_NN_jpe,                 // Jump if Parity Even (PF=1)
	STARS_NN_jpo,                 // Jump if Parity Odd  (PF=0)
	STARS_NN_js,                  // Jump if Sign (SF=1)
	STARS_NN_jz,                  // Jump if Zero (ZF=1)
	STARS_NN_jmp,                 // Jump
	STARS_NN_jmpfi,               // Indirect Far Jump
	STARS_NN_jmpni,               // Indirect Near Jump
	STARS_NN_jmpshort,            // Jump Short (not used)
	STARS_NN_lahf,                // Load Flags into AH Register
	STARS_NN_lar,                 // Load Access Right Byte
	STARS_NN_lea,                 // Load Effective Address
	STARS_NN_leavew,              // High Level Procedure Exit
	STARS_NN_leave,               // High Level Procedure Exit
	STARS_NN_leaved,              // High Level Procedure Exit
	STARS_NN_leaveq,              // High Level Procedure Exit
	STARS_NN_lgdt,                // Load Global Descriptor Table Register
	STARS_NN_lidt,                // Load Interrupt Descriptor Table Register
	STARS_NN_lgs,                 // Load Full Pointer to GS:xx
	STARS_NN_lss,                 // Load Full Pointer to SS:xx
	STARS_NN_lds,                 // Load Full Pointer to DS:xx
	STARS_NN_les,                 // Load Full Pointer to ES:xx
	STARS_NN_lfs,                 // Load Full Pointer to FS:xx
	STARS_NN_lldt,                // Load Local Descriptor Table Register
	STARS_NN_lmsw,                // Load Machine Status Word
	STARS_NN_lock,                // Assert LOCK# Signal Prefix
	STARS_NN_lods,                // Load String
	STARS_NN_loopw,               // Loop while ECX != 0
	STARS_NN_loop,                // Loop while CX != 0
	STARS_NN_loopd,               // Loop while ECX != 0
	STARS_NN_loopq,               // Loop while RCX != 0
	STARS_NN_loopwe,              // Loop while CX != 0 and ZF=1
	STARS_NN_loope,               // Loop while rCX != 0 and ZF=1
	STARS_NN_loopde,              // Loop while ECX != 0 and ZF=1
	STARS_NN_loopqe,              // Loop while RCX != 0 and ZF=1
	STARS_NN_loopwne,             // Loop while CX != 0 and ZF=0
	STARS_NN_loopne,              // Loop while rCX != 0 and ZF=0
	STARS_NN_loopdne,             // Loop while ECX != 0 and ZF=0
	STARS_NN_loopqne,             // Loop while RCX != 0 and ZF=0
	STARS_NN_lsl,                 // Load Segment Limit
	STARS_NN_ltr,                 // Load Task Register
	STARS_NN_mov,                 // Move Data
	STARS_NN_movsp,               // Move to/from Special Registers
	STARS_NN_movs,                // Move Byte(s) from String to String
	STARS_NN_movsx,               // Move with Sign-Extend
	STARS_NN_movzx,               // Move with Zero-Extend
	STARS_NN_mul,                 // Unsigned Multiplication of AL or AX
	STARS_NN_neg,                 // Two's Complement Negation
	STARS_NN_nop,                 // No Operation
	STARS_NN_not,                 // One's Complement Negation
	STARS_NN_or,                  // Logical Inclusive OR
	STARS_NN_out,                 // Output to Port
	STARS_NN_outs,                // Output Byte(s) to Port
	STARS_NN_pop,                 // Pop a word from the Stack
	STARS_NN_popaw,               // Pop all General Registers
	STARS_NN_popa,                // Pop all General Registers
	STARS_NN_popad,               // Pop all General Registers (use32)
	STARS_NN_popaq,               // Pop all General Registers (use64)
	STARS_NN_popfw,               // Pop Stack into Flags Register
	STARS_NN_popf,                // Pop Stack into Flags Register
	STARS_NN_popfd,               // Pop Stack into Eflags Register
	STARS_NN_popfq,               // Pop Stack into Rflags Register
	STARS_NN_push,                // Push Operand onto the Stack
	STARS_NN_pushaw,              // Push all General Registers
	STARS_NN_pusha,               // Push all General Registers
	STARS_NN_pushad,              // Push all General Registers (use32)
	STARS_NN_pushaq,              // Push all General Registers (use64)
	STARS_NN_pushfw,              // Push Flags Register onto the Stack
	STARS_NN_pushf,               // Push Flags Register onto the Stack
	STARS_NN_pushfd,              // Push Flags Register onto the Stack (use32)
	STARS_NN_pushfq,              // Push Flags Register onto the Stack (use64)
	STARS_NN_rcl,                 // Rotate Through Carry Left
	STARS_NN_rcr,                 // Rotate Through Carry Right
	STARS_NN_rol,                 // Rotate Left
	STARS_NN_ror,                 // Rotate Right
	STARS_NN_rep,                 // Repeat String Operation
	STARS_NN_repe,                // Repeat String Operation while ZF=1
	STARS_NN_repne,               // Repeat String Operation while ZF=0
	STARS_NN_retn,                // Return Near from Procedure
	STARS_NN_retf,                // Return Far from Procedure
	STARS_NN_sahf,                // Store AH into Flags Register
	STARS_NN_sal,                 // Shift Arithmetic Left
	STARS_NN_sar,                 // Shift Arithmetic Right
	STARS_NN_shl,                 // Shift Logical Left
	STARS_NN_shr,                 // Shift Logical Right
	STARS_NN_sbb,                 // Integer Subtraction with Borrow
	STARS_NN_scas,                // Compare String
	STARS_NN_seta,                // Set Byte if Above (CF=0 & ZF=0)
	STARS_NN_setae,               // Set Byte if Above or Equal (CF=0)
	STARS_NN_setb,                // Set Byte if Below (CF=1)
	STARS_NN_setbe,               // Set Byte if Below or Equal (CF=1 | ZF=1)
	STARS_NN_setc,                // Set Byte if Carry (CF=1)
	STARS_NN_sete,                // Set Byte if Equal (ZF=1)
	STARS_NN_setg,                // Set Byte if Greater (ZF=0 & SF=OF)
	STARS_NN_setge,               // Set Byte if Greater or Equal (SF=OF)
	STARS_NN_setl,                // Set Byte if Less (SF!=OF)
	STARS_NN_setle,               // Set Byte if Less or Equal (ZF=1 | SF!=OF)
	STARS_NN_setna,               // Set Byte if Not Above (CF=1 | ZF=1)
	STARS_NN_setnae,              // Set Byte if Not Above or Equal (CF=1)
	STARS_NN_setnb,               // Set Byte if Not Below (CF=0)
	STARS_NN_setnbe,              // Set Byte if Not Below or Equal (CF=0 & ZF=0)
	STARS_NN_setnc,               // Set Byte if Not Carry (CF=0)
	STARS_NN_setne,               // Set Byte if Not Equal (ZF=0)
	STARS_NN_setng,               // Set Byte if Not Greater (ZF=1 | SF!=OF)
	STARS_NN_setnge,              // Set Byte if Not Greater or Equal (ZF=1)
	STARS_NN_setnl,               // Set Byte if Not Less (SF=OF)
	STARS_NN_setnle,              // Set Byte if Not Less or Equal (ZF=0 & SF=OF)
	STARS_NN_setno,               // Set Byte if Not Overflow (OF=0)
	STARS_NN_setnp,               // Set Byte if Not Parity (PF=0)
	STARS_NN_setns,               // Set Byte if Not Sign (SF=0)
	STARS_NN_setnz,               // Set Byte if Not Zero (ZF=0)
	STARS_NN_seto,                // Set Byte if Overflow (OF=1)
	STARS_NN_setp,                // Set Byte if Parity (PF=1)
	STARS_NN_setpe,               // Set Byte if Parity Even (PF=1)
	STARS_NN_setpo,               // Set Byte if Parity Odd  (PF=0)
	STARS_NN_sets,                // Set Byte if Sign (SF=1)
	STARS_NN_setz,                // Set Byte if Zero (ZF=1)
	STARS_NN_sgdt,                // Store Global Descriptor Table Register
	STARS_NN_sidt,                // Store Interrupt Descriptor Table Register
	STARS_NN_shld,                // Double Precision Shift Left
	STARS_NN_shrd,                // Double Precision Shift Right
	STARS_NN_sldt,                // Store Local Descriptor Table Register
	STARS_NN_smsw,                // Store Machine Status Word
	STARS_NN_stc,                 // Set Carry Flag
	STARS_NN_std,                 // Set Direction Flag
	STARS_NN_sti,                 // Set Interrupt Flag
	STARS_NN_stos,                // Store String
	STARS_NN_str,                 // Store Task Register
	STARS_NN_sub,                 // Integer Subtraction
	STARS_NN_test,                // Logical Compare
	STARS_NN_verr,                // Verify a Segment for Reading
	STARS_NN_verw,                // Verify a Segment for Writing
	STARS_NN_wait,                // Wait until BUSY# Pin is Inactive (HIGH)
	STARS_NN_xchg,                // Exchange Register/Memory with Register
	STARS_NN_xlat,                // Table Lookup Translation
	STARS_NN_xor,                 // Logical Exclusive OR

	//
	//      486 instructions
	//

	STARS_NN_cmpxchg,             // Compare and Exchange
	STARS_NN_bswap,               // Swap bits in EAX
	STARS_NN_xadd,                // t<-dest; dest<-src+dest; src<-t
	STARS_NN_invd,                // Invalidate Data Cache
	STARS_NN_wbinvd,              // Invalidate Data Cache (write changes)
	STARS_NN_invlpg,              // Invalidate TLB entry

	//
	//      Pentium instructions
	//

	STARS_NN_rdmsr,               // Read Machine Status Register
	STARS_NN_wrmsr,               // Write Machine Status Register
	STARS_NN_cpuid,               // Get CPU ID
	STARS_NN_cmpxchg8b,           // Compare and Exchange Eight Bytes
	STARS_NN_rdtsc,               // Read Time Stamp Counter
	STARS_NN_rsm,                 // Resume from System Management Mode

	//
	//      Pentium Pro instructions
	//

	STARS_NN_cmova,               // Move if Above (CF=0 & ZF=0)
	STARS_NN_cmovb,               // Move if Below (CF=1)
	STARS_NN_cmovbe,              // Move if Below or Equal (CF=1 | ZF=1)
	STARS_NN_cmovg,               // Move if Greater (ZF=0 & SF=OF)
	STARS_NN_cmovge,              // Move if Greater or Equal (SF=OF)
	STARS_NN_cmovl,               // Move if Less (SF!=OF)
	STARS_NN_cmovle,              // Move if Less or Equal (ZF=1 | SF!=OF)
	STARS_NN_cmovnb,              // Move if Not Below (CF=0)
	STARS_NN_cmovno,              // Move if Not Overflow (OF=0)
	STARS_NN_cmovnp,              // Move if Not Parity (PF=0)
	STARS_NN_cmovns,              // Move if Not Sign (SF=0)
	STARS_NN_cmovnz,              // Move if Not Zero (ZF=0)
	STARS_NN_cmovo,               // Move if Overflow (OF=1)
	STARS_NN_cmovp,               // Move if Parity (PF=1)
	STARS_NN_cmovs,               // Move if Sign (SF=1)
	STARS_NN_cmovz,               // Move if Zero (ZF=1)
	STARS_NN_fcmovb,              // Floating Move if Below
	STARS_NN_fcmove,              // Floating Move if Equal
	STARS_NN_fcmovbe,             // Floating Move if Below or Equal
	STARS_NN_fcmovu,              // Floating Move if Unordered
	STARS_NN_fcmovnb,             // Floating Move if Not Below
	STARS_NN_fcmovne,             // Floating Move if Not Equal
	STARS_NN_fcmovnbe,            // Floating Move if Not Below or Equal
	STARS_NN_fcmovnu,             // Floating Move if Not Unordered
	STARS_NN_fcomi,               // FP Compare, result in EFLAGS
	STARS_NN_fucomi,              // FP Unordered Compare, result in EFLAGS
	STARS_NN_fcomip,              // FP Compare, result in EFLAGS, pop stack
	STARS_NN_fucomip,             // FP Unordered Compare, result in EFLAGS, pop stack
	STARS_NN_rdpmc,               // Read Performance Monitor Counter

	//
	//      FPP instructuions
	//

	STARS_NN_fld,                 // Load Real
	STARS_NN_fst,                 // Store Real
	STARS_NN_fstp,                // Store Real and Pop
	STARS_NN_fxch,                // Exchange Registers
	STARS_NN_fild,                // Load Integer
	STARS_NN_fist,                // Store Integer
	STARS_NN_fistp,               // Store Integer and Pop
	STARS_NN_fbld,                // Load BCD
	STARS_NN_fbstp,               // Store BCD and Pop
	STARS_NN_fadd,                // Add Real
	STARS_NN_faddp,               // Add Real and Pop
	STARS_NN_fiadd,               // Add Integer
	STARS_NN_fsub,                // Subtract Real
	STARS_NN_fsubp,               // Subtract Real and Pop
	STARS_NN_fisub,               // Subtract Integer
	STARS_NN_fsubr,               // Subtract Real Reversed
	STARS_NN_fsubrp,              // Subtract Real Reversed and Pop
	STARS_NN_fisubr,              // Subtract Integer Reversed
	STARS_NN_fmul,                // Multiply Real
	STARS_NN_fmulp,               // Multiply Real and Pop
	STARS_NN_fimul,               // Multiply Integer
	STARS_NN_fdiv,                // Divide Real
	STARS_NN_fdivp,               // Divide Real and Pop
	STARS_NN_fidiv,               // Divide Integer
	STARS_NN_fdivr,               // Divide Real Reversed
	STARS_NN_fdivrp,              // Divide Real Reversed and Pop
	STARS_NN_fidivr,              // Divide Integer Reversed
	STARS_NN_fsqrt,               // Square Root
	STARS_NN_fscale,              // Scale:  st(0) <- st(0) * 2^st(1)
	STARS_NN_fprem,               // Partial Remainder
	STARS_NN_frndint,             // Round to Integer
	STARS_NN_fxtract,             // Extract exponent and significand
	STARS_NN_fabs,                // Absolute value
	STARS_NN_fchs,                // Change Sign
	STARS_NN_fcom,                // Compare Real
	STARS_NN_fcomp,               // Compare Real and Pop
	STARS_NN_fcompp,              // Compare Real and Pop Twice
	STARS_NN_ficom,               // Compare Integer
	STARS_NN_ficomp,              // Compare Integer and Pop
	STARS_NN_ftst,                // Test
	STARS_NN_fxam,                // Examine
	STARS_NN_fptan,               // Partial tangent
	STARS_NN_fpatan,              // Partial arctangent
	STARS_NN_f2xm1,               // 2^x - 1
	STARS_NN_fyl2x,               // Y * lg2(X)
	STARS_NN_fyl2xp1,             // Y * lg2(X+1)
	STARS_NN_fldz,                // Load +0.0
	STARS_NN_fld1,                // Load +1.0
	STARS_NN_fldpi,               // Load PI=3.14...
	STARS_NN_fldl2t,              // Load lg2(10)
	STARS_NN_fldl2e,              // Load lg2(e)
	STARS_NN_fldlg2,              // Load lg10(2)
	STARS_NN_fldln2,              // Load ln(2)
	STARS_NN_finit,               // Initialize Processor
	STARS_NN_fninit,              // Initialize Processor (no wait)
	STARS_NN_fsetpm,              // Set Protected Mode
	STARS_NN_fldcw,               // Load Control Word
	STARS_NN_fstcw,               // Store Control Word
	STARS_NN_fnstcw,              // Store Control Word (no wait)
	STARS_NN_fstsw,               // Store Status Word
	STARS_NN_fnstsw,              // Store Status Word (no wait)
	STARS_NN_fclex,               // Clear Exceptions
	STARS_NN_fnclex,              // Clear Exceptions (no wait)
	STARS_NN_fstenv,              // Store Environment
	STARS_NN_fnstenv,             // Store Environment (no wait)
	STARS_NN_fldenv,              // Load Environment
	STARS_NN_fsave,               // Save State
	STARS_NN_fnsave,              // Save State (no wait)
	STARS_NN_frstor,              // Restore State
	STARS_NN_fincstp,             // Increment Stack Pointer
	STARS_NN_fdecstp,             // Decrement Stack Pointer
	STARS_NN_ffree,               // Free Register
	STARS_NN_fnop,                // No Operation
	STARS_NN_feni,                // (8087 only)
	STARS_NN_fneni,               // (no wait) (8087 only)
	STARS_NN_fdisi,               // (8087 only)
	STARS_NN_fndisi,              // (no wait) (8087 only)

	//
	//      80387 instructions
	//

	STARS_NN_fprem1,              // Partial Remainder ( < half )
	STARS_NN_fsincos,             // t<-cos(st); st<-sin(st); push t
	STARS_NN_fsin,                // Sine
	STARS_NN_fcos,                // Cosine
	STARS_NN_fucom,               // Compare Unordered Real
	STARS_NN_fucomp,              // Compare Unordered Real and Pop
	STARS_NN_fucompp,             // Compare Unordered Real and Pop Twice

	//
	//      Instructions added 28.02.96
	//

	STARS_NN_setalc,              // Set AL to Carry Flag
	STARS_NN_svdc,                // Save Register and Descriptor
	STARS_NN_rsdc,                // Restore Register and Descriptor
	STARS_NN_svldt,               // Save LDTR and Descriptor
	STARS_NN_rsldt,               // Restore LDTR and Descriptor
	STARS_NN_svts,                // Save TR and Descriptor
	STARS_NN_rsts,                // Restore TR and Descriptor
	STARS_NN_icebp,               // ICE Break Point
	STARS_NN_loadall,             // Load the entire CPU state from ES:EDI

	//
	//      MMX instructions
	//

	STARS_NN_emms,                // Empty MMX state
	STARS_NN_movd,                // Move 32 bits
	STARS_NN_movq,                // Move 64 bits
	STARS_NN_packsswb,            // Pack with Signed Saturation (Word->Byte)
	STARS_NN_packssdw,            // Pack with Signed Saturation (Dword->Word)
	STARS_NN_packuswb,            // Pack with Unsigned Saturation (Word->Byte)
	STARS_NN_paddb,               // Packed Add Byte
	STARS_NN_paddw,               // Packed Add Word
	STARS_NN_paddd,               // Packed Add Dword
	STARS_NN_paddsb,              // Packed Add with Saturation (Byte)
	STARS_NN_paddsw,              // Packed Add with Saturation (Word)
	STARS_NN_paddusb,             // Packed Add Unsigned with Saturation (Byte)
	STARS_NN_paddusw,             // Packed Add Unsigned with Saturation (Word)
	STARS_NN_pand,                // Bitwise Logical And
	STARS_NN_pandn,               // Bitwise Logical And Not
	STARS_NN_pcmpeqb,             // Packed Compare for Equal (Byte)
	STARS_NN_pcmpeqw,             // Packed Compare for Equal (Word)
	STARS_NN_pcmpeqd,             // Packed Compare for Equal (Dword)
	STARS_NN_pcmpgtb,             // Packed Compare for Greater Than (Byte)
	STARS_NN_pcmpgtw,             // Packed Compare for Greater Than (Word)
	STARS_NN_pcmpgtd,             // Packed Compare for Greater Than (Dword)
	STARS_NN_pmaddwd,             // Packed Multiply and Add
	STARS_NN_pmulhw,              // Packed Multiply High
	STARS_NN_pmullw,              // Packed Multiply Low
	STARS_NN_por,                 // Bitwise Logical Or
	STARS_NN_psllw,               // Packed Shift Left Logical (Word)
	STARS_NN_pslld,               // Packed Shift Left Logical (Dword)
	STARS_NN_psllq,               // Packed Shift Left Logical (Qword)
	STARS_NN_psraw,               // Packed Shift Right Arithmetic (Word)
	STARS_NN_psrad,               // Packed Shift Right Arithmetic (Dword)
	STARS_NN_psrlw,               // Packed Shift Right Logical (Word)
	STARS_NN_psrld,               // Packed Shift Right Logical (Dword)
	STARS_NN_psrlq,               // Packed Shift Right Logical (Qword)
	STARS_NN_psubb,               // Packed Subtract Byte
	STARS_NN_psubw,               // Packed Subtract Word
	STARS_NN_psubd,               // Packed Subtract Dword
	STARS_NN_psubsb,              // Packed Subtract with Saturation (Byte)
	STARS_NN_psubsw,              // Packed Subtract with Saturation (Word)
	STARS_NN_psubusb,             // Packed Subtract Unsigned with Saturation (Byte)
	STARS_NN_psubusw,             // Packed Subtract Unsigned with Saturation (Word)
	STARS_NN_punpckhbw,           // Unpack High Packed Data (Byte->Word)
	STARS_NN_punpckhwd,           // Unpack High Packed Data (Word->Dword)
	STARS_NN_punpckhdq,           // Unpack High Packed Data (Dword->Qword)
	STARS_NN_punpcklbw,           // Unpack Low Packed Data (Byte->Word)
	STARS_NN_punpcklwd,           // Unpack Low Packed Data (Word->Dword)
	STARS_NN_punpckldq,           // Unpack Low Packed Data (Dword->Qword)
	STARS_NN_pxor,                // Bitwise Logical Exclusive Or

	//
	//      Undocumented Deschutes processor instructions
	//

	STARS_NN_fxsave,              // Fast save FP context
	STARS_NN_fxrstor,             // Fast restore FP context

	//      Pentium II instructions

	STARS_NN_sysenter,            // Fast Transition to System Call Entry Point
	STARS_NN_sysexit,             // Fast Transition from System Call Entry Point

	//      3DNow! instructions

	STARS_NN_pavgusb,             // Packed 8-bit Unsigned Integer Averaging
	STARS_NN_pfadd,               // Packed Floating-Point Addition
	STARS_NN_pfsub,               // Packed Floating-Point Subtraction
	STARS_NN_pfsubr,              // Packed Floating-Point Reverse Subtraction
	STARS_NN_pfacc,               // Packed Floating-Point Accumulate
	STARS_NN_pfcmpge,             // Packed Floating-Point Comparison, Greater or Equal
	STARS_NN_pfcmpgt,             // Packed Floating-Point Comparison, Greater
	STARS_NN_pfcmpeq,             // Packed Floating-Point Comparison, Equal
	STARS_NN_pfmin,               // Packed Floating-Point Minimum
	STARS_NN_pfmax,               // Packed Floating-Point Maximum
	STARS_NN_pi2fd,               // Packed 32-bit Integer to Floating-Point
	STARS_NN_pf2id,               // Packed Floating-Point to 32-bit Integer
	STARS_NN_pfrcp,               // Packed Floating-Point Reciprocal Approximation
	STARS_NN_pfrsqrt,             // Packed Floating-Point Reciprocal Square Root Approximation
	STARS_NN_pfmul,               // Packed Floating-Point Multiplication
	STARS_NN_pfrcpit1,            // Packed Floating-Point Reciprocal First Iteration Step
	STARS_NN_pfrsqit1,            // Packed Floating-Point Reciprocal Square Root First Iteration Step
	STARS_NN_pfrcpit2,            // Packed Floating-Point Reciprocal Second Iteration Step
	STARS_NN_pmulhrw,             // Packed Floating-Point 16-bit Integer Multiply with rounding
	STARS_NN_femms,               // Faster entry/exit of the MMX or floating-point state
	STARS_NN_prefetch,            // Prefetch at least a 32-byte line into L1 data cache
	STARS_NN_prefetchw,           // Prefetch processor cache line into L1 data cache (mark as modified)


	//      Pentium III instructions

	STARS_NN_addps,               // Packed Single-FP Add
	STARS_NN_addss,               // Scalar Single-FP Add
	STARS_NN_andnps,              // Bitwise Logical And Not for Single-FP
	STARS_NN_andps,               // Bitwise Logical And for Single-FP
	STARS_NN_cmpps,               // Packed Single-FP Compare
	STARS_NN_cmpss,               // Scalar Single-FP Compare
	STARS_NN_comiss,              // Scalar Ordered Single-FP Compare and Set EFLAGS
	STARS_NN_cvtpi2ps,            // Packed signed INT32 to Packed Single-FP conversion
	STARS_NN_cvtps2pi,            // Packed Single-FP to Packed INT32 conversion
	STARS_NN_cvtsi2ss,            // Scalar signed INT32 to Single-FP conversion
	STARS_NN_cvtss2si,            // Scalar Single-FP to signed INT32 conversion
	STARS_NN_cvttps2pi,           // Packed Single-FP to Packed INT32 conversion (truncate)
	STARS_NN_cvttss2si,           // Scalar Single-FP to signed INT32 conversion (truncate)
	STARS_NN_divps,               // Packed Single-FP Divide
	STARS_NN_divss,               // Scalar Single-FP Divide
	STARS_NN_ldmxcsr,             // Load Streaming SIMD Extensions Technology Control/Status Register
	STARS_NN_maxps,               // Packed Single-FP Maximum
	STARS_NN_maxss,               // Scalar Single-FP Maximum
	STARS_NN_minps,               // Packed Single-FP Minimum
	STARS_NN_minss,               // Scalar Single-FP Minimum
	STARS_NN_movaps,              // Move Aligned Four Packed Single-FP
	STARS_NN_movhlps,             // Move High to Low Packed Single-FP
	STARS_NN_movhps,              // Move High Packed Single-FP
	STARS_NN_movlhps,             // Move Low to High Packed Single-FP
	STARS_NN_movlps,              // Move Low Packed Single-FP
	STARS_NN_movmskps,            // Move Mask to Register
	STARS_NN_movss,               // Move Scalar Single-FP
	STARS_NN_movups,              // Move Unaligned Four Packed Single-FP
	STARS_NN_mulps,               // Packed Single-FP Multiply
	STARS_NN_mulss,               // Scalar Single-FP Multiply
	STARS_NN_orps,                // Bitwise Logical OR for Single-FP Data
	STARS_NN_rcpps,               // Packed Single-FP Reciprocal
	STARS_NN_rcpss,               // Scalar Single-FP Reciprocal
	STARS_NN_rsqrtps,             // Packed Single-FP Square Root Reciprocal
	STARS_NN_rsqrtss,             // Scalar Single-FP Square Root Reciprocal
	STARS_NN_shufps,              // Shuffle Single-FP
	STARS_NN_sqrtps,              // Packed Single-FP Square Root
	STARS_NN_sqrtss,              // Scalar Single-FP Square Root
	STARS_NN_stmxcsr,             // Store Streaming SIMD Extensions Technology Control/Status Register
	STARS_NN_subps,               // Packed Single-FP Subtract
	STARS_NN_subss,               // Scalar Single-FP Subtract
	STARS_NN_ucomiss,             // Scalar Unordered Single-FP Compare and Set EFLAGS
	STARS_NN_unpckhps,            // Unpack High Packed Single-FP Data
	STARS_NN_unpcklps,            // Unpack Low Packed Single-FP Data
	STARS_NN_xorps,               // Bitwise Logical XOR for Single-FP Data
	STARS_NN_pavgb,               // Packed Average (Byte)
	STARS_NN_pavgw,               // Packed Average (Word)
	STARS_NN_pextrw,              // Extract Word
	STARS_NN_pinsrw,              // Insert Word
	STARS_NN_pmaxsw,              // Packed Signed Integer Word Maximum
	STARS_NN_pmaxub,              // Packed Unsigned Integer Byte Maximum
	STARS_NN_pminsw,              // Packed Signed Integer Word Minimum
	STARS_NN_pminub,              // Packed Unsigned Integer Byte Minimum
	STARS_NN_pmovmskb,            // Move Byte Mask to Integer
	STARS_NN_pmulhuw,             // Packed Multiply High Unsigned
	STARS_NN_psadbw,              // Packed Sum of Absolute Differences
	STARS_NN_pshufw,              // Packed Shuffle Word
	STARS_NN_maskmovq,            // Byte Mask write
	STARS_NN_movntps,             // Move Aligned Four Packed Single-FP Non Temporal
	STARS_NN_movntq,              // Move 64 Bits Non Temporal
	STARS_NN_prefetcht0,          // Prefetch to all cache levels
	STARS_NN_prefetcht1,          // Prefetch to all cache levels
	STARS_NN_prefetcht2,          // Prefetch to L2 cache
	STARS_NN_prefetchnta,         // Prefetch to L1 cache
	STARS_NN_sfence,              // Store Fence

	// Pentium III Pseudo instructions

	STARS_NN_cmpeqps,             // Packed Single-FP Compare EQ
	STARS_NN_cmpltps,             // Packed Single-FP Compare LT
	STARS_NN_cmpleps,             // Packed Single-FP Compare LE
	STARS_NN_cmpunordps,          // Packed Single-FP Compare UNORD
	STARS_NN_cmpneqps,            // Packed Single-FP Compare NOT EQ
	STARS_NN_cmpnltps,            // Packed Single-FP Compare NOT LT
	STARS_NN_cmpnleps,            // Packed Single-FP Compare NOT LE
	STARS_NN_cmpordps,            // Packed Single-FP Compare ORDERED
	STARS_NN_cmpeqss,             // Scalar Single-FP Compare EQ
	STARS_NN_cmpltss,             // Scalar Single-FP Compare LT
	STARS_NN_cmpless,             // Scalar Single-FP Compare LE
	STARS_NN_cmpunordss,          // Scalar Single-FP Compare UNORD
	STARS_NN_cmpneqss,            // Scalar Single-FP Compare NOT EQ
	STARS_NN_cmpnltss,            // Scalar Single-FP Compare NOT LT
	STARS_NN_cmpnless,            // Scalar Single-FP Compare NOT LE
	STARS_NN_cmpordss,            // Scalar Single-FP Compare ORDERED

	// AMD K7 instructions

	STARS_NN_pf2iw,               // Packed Floating-Point to Integer with Sign Extend
	STARS_NN_pfnacc,              // Packed Floating-Point Negative Accumulate
	STARS_NN_pfpnacc,             // Packed Floating-Point Mixed Positive-Negative Accumulate
	STARS_NN_pi2fw,               // Packed 16-bit Integer to Floating-Point
	STARS_NN_pswapd,              // Packed Swap Double Word

	// Undocumented FP instructions (thanks to norbert.juffa@adm.com)

	STARS_NN_fstp1,               // Alias of Store Real and Pop
	STARS_NN_fcom2,               // Alias of Compare Real
	STARS_NN_fcomp3,              // Alias of Compare Real and Pop
	STARS_NN_fxch4,               // Alias of Exchange Registers
	STARS_NN_fcomp5,              // Alias of Compare Real and Pop
	STARS_NN_ffreep,              // Free Register and Pop
	STARS_NN_fxch7,               // Alias of Exchange Registers
	STARS_NN_fstp8,               // Alias of Store Real and Pop
	STARS_NN_fstp9,               // Alias of Store Real and Pop

	// Pentium 4 instructions

	STARS_NN_addpd,               // Add Packed Double-Precision Floating-Point Values
	STARS_NN_addsd,               // Add Scalar Double-Precision Floating-Point Values
	STARS_NN_andnpd,              // Bitwise Logical AND NOT of Packed Double-Precision Floating-Point Values
	STARS_NN_andpd,               // Bitwise Logical AND of Packed Double-Precision Floating-Point Values
	STARS_NN_clflush,             // Flush Cache Line
	STARS_NN_cmppd,               // Compare Packed Double-Precision Floating-Point Values
	STARS_NN_cmpsd,               // Compare Scalar Double-Precision Floating-Point Values
	STARS_NN_comisd,              // Compare Scalar Ordered Double-Precision Floating-Point Values and Set EFLAGS
	STARS_NN_cvtdq2pd,            // Convert Packed Doubleword Integers to Packed Single-Precision Floating-Point Values
	STARS_NN_cvtdq2ps,            // Convert Packed Doubleword Integers to Packed Double-Precision Floating-Point Values
	STARS_NN_cvtpd2dq,            // Convert Packed Double-Precision Floating-Point Values to Packed Doubleword Integers
	STARS_NN_cvtpd2pi,            // Convert Packed Double-Precision Floating-Point Values to Packed Doubleword Integers
	STARS_NN_cvtpd2ps,            // Convert Packed Double-Precision Floating-Point Values to Packed Single-Precision Floating-Point Values
	STARS_NN_cvtpi2pd,            // Convert Packed Doubleword Integers to Packed Double-Precision Floating-Point Values
	STARS_NN_cvtps2dq,            // Convert Packed Single-Precision Floating-Point Values to Packed Doubleword Integers
	STARS_NN_cvtps2pd,            // Convert Packed Single-Precision Floating-Point Values to Packed Double-Precision Floating-Point Values
	STARS_NN_cvtsd2si,            // Convert Scalar Double-Precision Floating-Point Value to Doubleword Integer
	STARS_NN_cvtsd2ss,            // Convert Scalar Double-Precision Floating-Point Value to Scalar Single-Precision Floating-Point Value
	STARS_NN_cvtsi2sd,            // Convert Doubleword Integer to Scalar Double-Precision Floating-Point Value
	STARS_NN_cvtss2sd,            // Convert Scalar Single-Precision Floating-Point Value to Scalar Double-Precision Floating-Point Value
	STARS_NN_cvttpd2dq,           // Convert With Truncation Packed Double-Precision Floating-Point Values to Packed Doubleword Integers
	STARS_NN_cvttpd2pi,           // Convert with Truncation Packed Double-Precision Floating-Point Values to Packed Doubleword Integers
	STARS_NN_cvttps2dq,           // Convert With Truncation Packed Single-Precision Floating-Point Values to Packed Doubleword Integers
	STARS_NN_cvttsd2si,           // Convert with Truncation Scalar Double-Precision Floating-Point Value to Doubleword Integer
	STARS_NN_divpd,               // Divide Packed Double-Precision Floating-Point Values
	STARS_NN_divsd,               // Divide Scalar Double-Precision Floating-Point Values
	STARS_NN_lfence,              // Load Fence
	STARS_NN_maskmovdqu,          // Store Selected Bytes of Double Quadword
	STARS_NN_maxpd,               // Return Maximum Packed Double-Precision Floating-Point Values
	STARS_NN_maxsd,               // Return Maximum Scalar Double-Precision Floating-Point Value
	STARS_NN_mfence,              // Memory Fence
	STARS_NN_minpd,               // Return Minimum Packed Double-Precision Floating-Point Values
	STARS_NN_minsd,               // Return Minimum Scalar Double-Precision Floating-Point Value
	STARS_NN_movapd,              // Move Aligned Packed Double-Precision Floating-Point Values
	STARS_NN_movdq2q,             // Move Quadword from XMM to MMX Register
	STARS_NN_movdqa,              // Move Aligned Double Quadword
	STARS_NN_movdqu,              // Move Unaligned Double Quadword
	STARS_NN_movhpd,              // Move High Packed Double-Precision Floating-Point Values
	STARS_NN_movlpd,              // Move Low Packed Double-Precision Floating-Point Values
	STARS_NN_movmskpd,            // Extract Packed Double-Precision Floating-Point Sign Mask
	STARS_NN_movntdq,             // Store Double Quadword Using Non-Temporal Hint
	STARS_NN_movnti,              // Store Doubleword Using Non-Temporal Hint
	STARS_NN_movntpd,             // Store Packed Double-Precision Floating-Point Values Using Non-Temporal Hint
	STARS_NN_movq2dq,             // Move Quadword from MMX to XMM Register
	STARS_NN_movsd,               // Move Scalar Double-Precision Floating-Point Values
	STARS_NN_movupd,              // Move Unaligned Packed Double-Precision Floating-Point Values
	STARS_NN_mulpd,               // Multiply Packed Double-Precision Floating-Point Values
	STARS_NN_mulsd,               // Multiply Scalar Double-Precision Floating-Point Values
	STARS_NN_orpd,                // Bitwise Logical OR of Double-Precision Floating-Point Values
	STARS_NN_paddq,               // Add Packed Quadword Integers
	STARS_NN_pause,               // Spin Loop Hint
	STARS_NN_pmuludq,             // Multiply Packed Unsigned Doubleword Integers
	STARS_NN_pshufd,              // Shuffle Packed Doublewords
	STARS_NN_pshufhw,             // Shuffle Packed High Words
	STARS_NN_pshuflw,             // Shuffle Packed Low Words
	STARS_NN_pslldq,              // Shift Double Quadword Left Logical
	STARS_NN_psrldq,              // Shift Double Quadword Right Logical
	STARS_NN_psubq,               // Subtract Packed Quadword Integers
	STARS_NN_punpckhqdq,          // Unpack High Data
	STARS_NN_punpcklqdq,          // Unpack Low Data
	STARS_NN_shufpd,              // Shuffle Packed Double-Precision Floating-Point Values
	STARS_NN_sqrtpd,              // Compute Square Roots of Packed Double-Precision Floating-Point Values
	STARS_NN_sqrtsd,              // Compute Square Rootof Scalar Double-Precision Floating-Point Value
	STARS_NN_subpd,               // Subtract Packed Double-Precision Floating-Point Values
	STARS_NN_subsd,               // Subtract Scalar Double-Precision Floating-Point Values
	STARS_NN_ucomisd,             // Unordered Compare Scalar Ordered Double-Precision Floating-Point Values and Set EFLAGS
	STARS_NN_unpckhpd,            // Unpack and Interleave High Packed Double-Precision Floating-Point Values
	STARS_NN_unpcklpd,            // Unpack and Interleave Low Packed Double-Precision Floating-Point Values
	STARS_NN_xorpd,               // Bitwise Logical OR of Double-Precision Floating-Point Values

	// AMD syscall/sysret instructions

	STARS_NN_syscall,             // Low latency system call
	STARS_NN_sysret,              // Return from system call

	// AMD64 instructions

	STARS_NN_swapgs,              // Exchange GS base with KernelGSBase MSR

	// New Pentium instructions (SSE3)

	STARS_NN_movddup,             // Move One Double-FP and Duplicate
	STARS_NN_movshdup,            // Move Packed Single-FP High and Duplicate
	STARS_NN_movsldup,            // Move Packed Single-FP Low and Duplicate

	// Missing AMD64 instructions

	STARS_NN_movsxd,              // Move with Sign-Extend Doubleword
	STARS_NN_cmpxchg16b,          // Compare and Exchange 16 Bytes

	// SSE3 instructions

	STARS_NN_addsubpd,            // Add /Sub packed DP FP numbers
	STARS_NN_addsubps,            // Add /Sub packed SP FP numbers
	STARS_NN_haddpd,              // Add horizontally packed DP FP numbers
	STARS_NN_haddps,              // Add horizontally packed SP FP numbers
	STARS_NN_hsubpd,              // Sub horizontally packed DP FP numbers
	STARS_NN_hsubps,              // Sub horizontally packed SP FP numbers
	STARS_NN_monitor,             // Set up a linear address range to be monitored by hardware
	STARS_NN_mwait,               // Wait until write-back store performed within the range specified by the MONITOR instruction
	STARS_NN_fisttp,              // Store ST in intXX (chop) and pop
	STARS_NN_lddqu,               // Load unaligned integer 128-bit

	// SSSE3 instructions

	STARS_NN_psignb,              // Packed SIGN Byte
	STARS_NN_psignw,              // Packed SIGN Word
	STARS_NN_psignd,              // Packed SIGN Doubleword
	STARS_NN_pshufb,              // Packed Shuffle Bytes
	STARS_NN_pmulhrsw,            // Packed Multiply High with Round and Scale
	STARS_NN_pmaddubsw,           // Multiply and Add Packed Signed and Unsigned Bytes
	STARS_NN_phsubsw,             // Packed Horizontal Subtract and Saturate
	STARS_NN_phaddsw,             // Packed Horizontal Add and Saturate
	STARS_NN_phaddw,              // Packed Horizontal Add Word
	STARS_NN_phaddd,              // Packed Horizontal Add Doubleword
	STARS_NN_phsubw,              // Packed Horizontal Subtract Word
	STARS_NN_phsubd,              // Packed Horizontal Subtract Doubleword
	STARS_NN_palignr,             // Packed Align Right
	STARS_NN_pabsb,               // Packed Absolute Value Byte
	STARS_NN_pabsw,               // Packed Absolute Value Word
	STARS_NN_pabsd,               // Packed Absolute Value Doubleword

	// VMX instructions

	STARS_NN_vmcall,              // Call to VM Monitor
	STARS_NN_vmclear,             // Clear Virtual Machine Control Structure
	STARS_NN_vmlaunch,            // Launch Virtual Machine
	STARS_NN_vmresume,            // Resume Virtual Machine
	STARS_NN_vmptrld,             // Load Pointer to Virtual Machine Control Structure
	STARS_NN_vmptrst,             // Store Pointer to Virtual Machine Control Structure
	STARS_NN_vmread,              // Read Field from Virtual Machine Control Structure
	STARS_NN_vmwrite,             // Write Field from Virtual Machine Control Structure
	STARS_NN_vmxoff,              // Leave VMX Operation
	STARS_NN_vmxon,               // Enter VMX Operation

	// Undefined Instruction

	STARS_NN_ud2,                 // Undefined Instruction

	// Added with x86-64

	STARS_NN_rdtscp,              // Read Time-Stamp Counter and Processor ID

	// Geode LX 3DNow! extensions

	STARS_NN_pfrcpv,              // Reciprocal Approximation for a Pair of 32-bit Floats
	STARS_NN_pfrsqrtv,            // Reciprocal Square Root Approximation for a Pair of 32-bit Floats

	// SSE2 pseudoinstructions

	STARS_NN_cmpeqpd,             // Packed Double-FP Compare EQ
	STARS_NN_cmpltpd,             // Packed Double-FP Compare LT
	STARS_NN_cmplepd,             // Packed Double-FP Compare LE
	STARS_NN_cmpunordpd,          // Packed Double-FP Compare UNORD
	STARS_NN_cmpneqpd,            // Packed Double-FP Compare NOT EQ
	STARS_NN_cmpnltpd,            // Packed Double-FP Compare NOT LT
	STARS_NN_cmpnlepd,            // Packed Double-FP Compare NOT LE
	STARS_NN_cmpordpd,            // Packed Double-FP Compare ORDERED
	STARS_NN_cmpeqsd,             // Scalar Double-FP Compare EQ
	STARS_NN_cmpltsd,             // Scalar Double-FP Compare LT
	STARS_NN_cmplesd,             // Scalar Double-FP Compare LE
	STARS_NN_cmpunordsd,          // Scalar Double-FP Compare UNORD
	STARS_NN_cmpneqsd,            // Scalar Double-FP Compare NOT EQ
	STARS_NN_cmpnltsd,            // Scalar Double-FP Compare NOT LT
	STARS_NN_cmpnlesd,            // Scalar Double-FP Compare NOT LE
	STARS_NN_cmpordsd,            // Scalar Double-FP Compare ORDERED

	// SSSE4.1 instructions

	STARS_NN_blendpd,              // Blend Packed Double Precision Floating-Point Values
	STARS_NN_blendps,              // Blend Packed Single Precision Floating-Point Values
	STARS_NN_blendvpd,             // Variable Blend Packed Double Precision Floating-Point Values
	STARS_NN_blendvps,             // Variable Blend Packed Single Precision Floating-Point Values
	STARS_NN_dppd,                 // Dot Product of Packed Double Precision Floating-Point Values
	STARS_NN_dpps,                 // Dot Product of Packed Single Precision Floating-Point Values
	STARS_NN_extractps,            // Extract Packed Single Precision Floating-Point Value
	STARS_NN_insertps,             // Insert Packed Single Precision Floating-Point Value
	STARS_NN_movntdqa,             // Load Double Quadword Non-Temporal Aligned Hint
	STARS_NN_mpsadbw,              // Compute Multiple Packed Sums of Absolute Difference
	STARS_NN_packusdw,             // Pack with Unsigned Saturation
	STARS_NN_pblendvb,             // Variable Blend Packed Bytes
	STARS_NN_pblendw,              // Blend Packed Words
	STARS_NN_pcmpeqq,              // Compare Packed Qword Data for Equal
	STARS_NN_pextrb,               // Extract Byte
	STARS_NN_pextrd,               // Extract Dword
	STARS_NN_pextrq,               // Extract Qword
	STARS_NN_phminposuw,           // Packed Horizontal Word Minimum
	STARS_NN_pinsrb,               // Insert Byte
	STARS_NN_pinsrd,               // Insert Dword
	STARS_NN_pinsrq,               // Insert Qword
	STARS_NN_pmaxsb,               // Maximum of Packed Signed Byte Integers
	STARS_NN_pmaxsd,               // Maximum of Packed Signed Dword Integers
	STARS_NN_pmaxud,               // Maximum of Packed Unsigned Dword Integers
	STARS_NN_pmaxuw,               // Maximum of Packed Word Integers
	STARS_NN_pminsb,               // Minimum of Packed Signed Byte Integers
	STARS_NN_pminsd,               // Minimum of Packed Signed Dword Integers
	STARS_NN_pminud,               // Minimum of Packed Unsigned Dword Integers
	STARS_NN_pminuw,               // Minimum of Packed Word Integers
	STARS_NN_pmovsxbw,             // Packed Move with Sign Extend
	STARS_NN_pmovsxbd,             // Packed Move with Sign Extend
	STARS_NN_pmovsxbq,             // Packed Move with Sign Extend
	STARS_NN_pmovsxwd,             // Packed Move with Sign Extend
	STARS_NN_pmovsxwq,             // Packed Move with Sign Extend
	STARS_NN_pmovsxdq,             // Packed Move with Sign Extend
	STARS_NN_pmovzxbw,             // Packed Move with Zero Extend
	STARS_NN_pmovzxbd,             // Packed Move with Zero Extend
	STARS_NN_pmovzxbq,             // Packed Move with Zero Extend
	STARS_NN_pmovzxwd,             // Packed Move with Zero Extend
	STARS_NN_pmovzxwq,             // Packed Move with Zero Extend
	STARS_NN_pmovzxdq,             // Packed Move with Zero Extend
	STARS_NN_pmuldq,               // Multiply Packed Signed Dword Integers
	STARS_NN_pmulld,               // Multiply Packed Signed Dword Integers and Store Low Result
	STARS_NN_ptest,                // Logical Compare
	STARS_NN_roundpd,              // Round Packed Double Precision Floating-Point Values
	STARS_NN_roundps,              // Round Packed Single Precision Floating-Point Values
	STARS_NN_roundsd,              // Round Scalar Double Precision Floating-Point Values
	STARS_NN_roundss,              // Round Scalar Single Precision Floating-Point Values

	// SSSE4.2 instructions

	STARS_NN_crc32,                // Accumulate CRC32 Value
	STARS_NN_pcmpestri,            // Packed Compare Explicit Length Strings, Return Index
	STARS_NN_pcmpestrm,            // Packed Compare Explicit Length Strings, Return Mask
	STARS_NN_pcmpistri,            // Packed Compare Implicit Length Strings, Return Index
	STARS_NN_pcmpistrm,            // Packed Compare Implicit Length Strings, Return Mask
	STARS_NN_pcmpgtq,              // Compare Packed Data for Greater Than
	STARS_NN_popcnt,               // Return the Count of Number of Bits Set to 1

	// AMD SSE4a instructions

	STARS_NN_extrq,                // Extract Field From Register
	STARS_NN_insertq,              // Insert Field
	STARS_NN_movntsd,              // Move Non-Temporal Scalar Double-Precision Floating-Point
	STARS_NN_movntss,              // Move Non-Temporal Scalar Single-Precision Floating-Point
	STARS_NN_lzcnt,                // Leading Zero Count

	// xsave/xrstor instructions

	STARS_NN_xgetbv,               // Get Value of Extended Control Register
	STARS_NN_xrstor,               // Restore Processor Extended States
	STARS_NN_xsave,                // Save Processor Extended States
	STARS_NN_xsetbv,               // Set Value of Extended Control Register

	// Intel Safer Mode Extensions (SMX)

	STARS_NN_getsec,               // Safer Mode Extensions (SMX) Instruction

	// AMD-V Virtualization ISA Extension

	STARS_NN_clgi,                 // Clear Global Interrupt Flag
	STARS_NN_invlpga,              // Invalidate TLB Entry in a Specified ASID
	STARS_NN_skinit,               // Secure Init and Jump with Attestation
	STARS_NN_stgi,                 // Set Global Interrupt Flag
	STARS_NN_vmexit,               // Stop Executing Guest, Begin Executing Host
	STARS_NN_vmload,               // Load State from VMCB
	STARS_NN_vmmcall,              // Call VMM
	STARS_NN_vmrun,                // Run Virtual Machine
	STARS_NN_vmsave,               // Save State to VMCB

	// VMX+ instructions

	STARS_NN_invept,               // Invalidate Translations Derived from EPT
	STARS_NN_invvpid,              // Invalidate Translations Based on VPID

	// Intel Atom instructions

	STARS_NN_movbe,                // Move Data After Swapping Bytes

	// Intel AES instructions

	STARS_NN_aesenc,                // Perform One Round of an AES Encryption Flow
	STARS_NN_aesenclast,            // Perform the Last Round of an AES Encryption Flow
	STARS_NN_aesdec,                // Perform One Round of an AES Decryption Flow
	STARS_NN_aesdeclast,            // Perform the Last Round of an AES Decryption Flow
	STARS_NN_aesimc,                // Perform the AES InvMixColumn Transformation
	STARS_NN_aeskeygenassist,       // AES Round Key Generation Assist

	// Carryless multiplication

	STARS_NN_pclmulqdq,            // Carry-Less Multiplication Quadword

	// Returns modifies by operand size prefixes

	STARS_NN_retnw,               // Return Near from Procedure (use16)
	STARS_NN_retnd,               // Return Near from Procedure (use32)
	STARS_NN_retnq,               // Return Near from Procedure (use64)
	STARS_NN_retfw,               // Return Far from Procedure (use16)
	STARS_NN_retfd,               // Return Far from Procedure (use32)
	STARS_NN_retfq,               // Return Far from Procedure (use64)

	// RDRAND support

	STARS_NN_rdrand,              // Read Random Number

	// new GPR instructions

	STARS_NN_adcx,                 // Unsigned Integer Addition of Two Operands with Carry Flag
	STARS_NN_adox,                 // Unsigned Integer Addition of Two Operands with Overflow Flag
	STARS_NN_andn,                 // Logical AND NOT
	STARS_NN_bextr,                // Bit Field Extract
	STARS_NN_blsi,                 // Extract Lowest Set Isolated Bit
	STARS_NN_blsmsk,               // Get Mask Up to Lowest Set Bit
	STARS_NN_blsr,                 // Reset Lowest Set Bit
	STARS_NN_bzhi,                 // Zero High Bits Starting with Specified Bit Position
	STARS_NN_clac,                 // Clear AC Flag in EFLAGS Register
	STARS_NN_mulx,                 // Unsigned Multiply Without Affecting Flags
	STARS_NN_pdep,                 // Parallel Bits Deposit
	STARS_NN_pext,                 // Parallel Bits Extract
	STARS_NN_rorx,                 // Rotate Right Logical Without Affecting Flags
	STARS_NN_sarx,                 // Shift Arithmetically Right Without Affecting Flags
	STARS_NN_shlx,                 // Shift Logically Left Without Affecting Flags
	STARS_NN_shrx,                 // Shift Logically Right Without Affecting Flags
	STARS_NN_stac,                 // Set AC Flag in EFLAGS Register
	STARS_NN_tzcnt,                // Count the Number of Trailing Zero Bits
	STARS_NN_xsaveopt,             // Save Processor Extended States Optimized
	STARS_NN_invpcid,              // Invalidate Processor Context ID
	STARS_NN_rdseed,               // Read Random Seed
	STARS_NN_rdfsbase,             // Read FS Segment Base
	STARS_NN_rdgsbase,             // Read GS Segment Base
	STARS_NN_wrfsbase,             // Write FS Segment Base
	STARS_NN_wrgsbase,             // Write GS Segment Base

	// new AVX instructions
	STARS_NN_vaddpd,               // Add Packed Double-Precision Floating-Point Values
	STARS_NN_vaddps,               // Packed Single-FP Add
	STARS_NN_vaddsd,               // Add Scalar Double-Precision Floating-Point Values
	STARS_NN_vaddss,               // Scalar Single-FP Add
	STARS_NN_vaddsubpd,            // Add /Sub packed DP FP numbers
	STARS_NN_vaddsubps,            // Add /Sub packed SP FP numbers
	STARS_NN_vaesdec,              // Perform One Round of an AES Decryption Flow
	STARS_NN_vaesdeclast,          // Perform the Last Round of an AES Decryption Flow
	STARS_NN_vaesenc,              // Perform One Round of an AES Encryption Flow
	STARS_NN_vaesenclast,          // Perform the Last Round of an AES Encryption Flow
	STARS_NN_vaesimc,              // Perform the AES InvMixColumn Transformation
	STARS_NN_vaeskeygenassist,     // AES Round Key Generation Assist
	STARS_NN_vandnpd,              // Bitwise Logical AND NOT of Packed Double-Precision Floating-Point Values
	STARS_NN_vandnps,              // Bitwise Logical And Not for Single-FP
	STARS_NN_vandpd,               // Bitwise Logical AND of Packed Double-Precision Floating-Point Values
	STARS_NN_vandps,               // Bitwise Logical And for Single-FP
	STARS_NN_vblendpd,             // Blend Packed Double Precision Floating-Point Values
	STARS_NN_vblendps,             // Blend Packed Single Precision Floating-Point Values
	STARS_NN_vblendvpd,            // Variable Blend Packed Double Precision Floating-Point Values
	STARS_NN_vblendvps,            // Variable Blend Packed Single Precision Floating-Point Values
	STARS_NN_vbroadcastf128,       // Broadcast 128 Bits of Floating-Point Data
	STARS_NN_vbroadcasti128,       // Broadcast 128 Bits of Integer Data
	STARS_NN_vbroadcastsd,         // Broadcast Double-Precision Floating-Point Element
	STARS_NN_vbroadcastss,         // Broadcast Single-Precision Floating-Point Element
	STARS_NN_vcmppd,               // Compare Packed Double-Precision Floating-Point Values
	STARS_NN_vcmpps,               // Packed Single-FP Compare
	STARS_NN_vcmpsd,               // Compare Scalar Double-Precision Floating-Point Values
	STARS_NN_vcmpss,               // Scalar Single-FP Compare
	STARS_NN_vcomisd,              // Compare Scalar Ordered Double-Precision Floating-Point Values and Set EFLAGS
	STARS_NN_vcomiss,              // Scalar Ordered Single-FP Compare and Set EFLAGS
	STARS_NN_vcvtdq2pd,            // Convert Packed Doubleword Integers to Packed Single-Precision Floating-Point Values
	STARS_NN_vcvtdq2ps,            // Convert Packed Doubleword Integers to Packed Double-Precision Floating-Point Values
	STARS_NN_vcvtpd2dq,            // Convert Packed Double-Precision Floating-Point Values to Packed Doubleword Integers
	STARS_NN_vcvtpd2ps,            // Convert Packed Double-Precision Floating-Point Values to Packed Single-Precision Floating-Point Values
	STARS_NN_vcvtph2ps,            // Convert 16-bit FP Values to Single-Precision FP Values
	STARS_NN_vcvtps2dq,            // Convert Packed Single-Precision Floating-Point Values to Packed Doubleword Integers
	STARS_NN_vcvtps2pd,            // Convert Packed Single-Precision Floating-Point Values to Packed Double-Precision Floating-Point Values
	STARS_NN_vcvtps2ph,            // Convert Single-Precision FP value to 16-bit FP value
	STARS_NN_vcvtsd2si,            // Convert Scalar Double-Precision Floating-Point Value to Doubleword Integer
	STARS_NN_vcvtsd2ss,            // Convert Scalar Double-Precision Floating-Point Value to Scalar Single-Precision Floating-Point Value
	STARS_NN_vcvtsi2sd,            // Convert Doubleword Integer to Scalar Double-Precision Floating-Point Value
	STARS_NN_vcvtsi2ss,            // Scalar signed INT32 to Single-FP conversion
	STARS_NN_vcvtss2sd,            // Convert Scalar Single-Precision Floating-Point Value to Scalar Double-Precision Floating-Point Value
	STARS_NN_vcvtss2si,            // Scalar Single-FP to signed INT32 conversion
	STARS_NN_vcvttpd2dq,           // Convert With Truncation Packed Double-Precision Floating-Point Values to Packed Doubleword Integers
	STARS_NN_vcvttps2dq,           // Convert With Truncation Packed Single-Precision Floating-Point Values to Packed Doubleword Integers
	STARS_NN_vcvttsd2si,           // Convert with Truncation Scalar Double-Precision Floating-Point Value to Doubleword Integer
	STARS_NN_vcvttss2si,           // Scalar Single-FP to signed INT32 conversion (truncate)
	STARS_NN_vdivpd,               // Divide Packed Double-Precision Floating-Point Values
	STARS_NN_vdivps,               // Packed Single-FP Divide
	STARS_NN_vdivsd,               // Divide Scalar Double-Precision Floating-Point Values
	STARS_NN_vdivss,               // Scalar Single-FP Divide
	STARS_NN_vdppd,                // Dot Product of Packed Double Precision Floating-Point Values
	STARS_NN_vdpps,                // Dot Product of Packed Single Precision Floating-Point Values
	STARS_NN_vextractf128,         // Extract Packed Floating-Point Values
	STARS_NN_vextracti128,         // Extract Packed Integer Values
	STARS_NN_vextractps,           // Extract Packed Floating-Point Values
	STARS_NN_vfmadd132pd,          // Fused Multiply-Add of Packed Double-Precision Floating-Point Values
	STARS_NN_vfmadd132ps,          // Fused Multiply-Add of Packed Single-Precision Floating-Point Values
	STARS_NN_vfmadd132sd,          // Fused Multiply-Add of Scalar Double-Precision Floating-Point Values
	STARS_NN_vfmadd132ss,          // Fused Multiply-Add of Scalar Single-Precision Floating-Point Values
	STARS_NN_vfmadd213pd,          // Fused Multiply-Add of Packed Double-Precision Floating-Point Values
	STARS_NN_vfmadd213ps,          // Fused Multiply-Add of Packed Single-Precision Floating-Point Values
	STARS_NN_vfmadd213sd,          // Fused Multiply-Add of Scalar Double-Precision Floating-Point Values
	STARS_NN_vfmadd213ss,          // Fused Multiply-Add of Scalar Single-Precision Floating-Point Values
	STARS_NN_vfmadd231pd,          // Fused Multiply-Add of Packed Double-Precision Floating-Point Values
	STARS_NN_vfmadd231ps,          // Fused Multiply-Add of Packed Single-Precision Floating-Point Values
	STARS_NN_vfmadd231sd,          // Fused Multiply-Add of Scalar Double-Precision Floating-Point Values
	STARS_NN_vfmadd231ss,          // Fused Multiply-Add of Scalar Single-Precision Floating-Point Values
	STARS_NN_vfmaddsub132pd,       // Fused Multiply-Alternating Add/Subtract of Packed Double-Precision Floating-Point Values
	STARS_NN_vfmaddsub132ps,       // Fused Multiply-Alternating Add/Subtract of Packed Single-Precision Floating-Point Values
	STARS_NN_vfmaddsub213pd,       // Fused Multiply-Alternating Add/Subtract of Packed Double-Precision Floating-Point Values
	STARS_NN_vfmaddsub213ps,       // Fused Multiply-Alternating Add/Subtract of Packed Single-Precision Floating-Point Values
	STARS_NN_vfmaddsub231pd,       // Fused Multiply-Alternating Add/Subtract of Packed Double-Precision Floating-Point Values
	STARS_NN_vfmaddsub231ps,       // Fused Multiply-Alternating Add/Subtract of Packed Single-Precision Floating-Point Values
	STARS_NN_vfmsub132pd,          // Fused Multiply-Subtract of Packed Double-Precision Floating-Point Values
	STARS_NN_vfmsub132ps,          // Fused Multiply-Subtract of Packed Single-Precision Floating-Point Values
	STARS_NN_vfmsub132sd,          // Fused Multiply-Subtract of Scalar Double-Precision Floating-Point Values
	STARS_NN_vfmsub132ss,          // Fused Multiply-Subtract of Scalar Single-Precision Floating-Point Values
	STARS_NN_vfmsub213pd,          // Fused Multiply-Subtract of Packed Double-Precision Floating-Point Values
	STARS_NN_vfmsub213ps,          // Fused Multiply-Subtract of Packed Single-Precision Floating-Point Values
	STARS_NN_vfmsub213sd,          // Fused Multiply-Subtract of Scalar Double-Precision Floating-Point Values
	STARS_NN_vfmsub213ss,          // Fused Multiply-Subtract of Scalar Single-Precision Floating-Point Values
	STARS_NN_vfmsub231pd,          // Fused Multiply-Subtract of Packed Double-Precision Floating-Point Values
	STARS_NN_vfmsub231ps,          // Fused Multiply-Subtract of Packed Single-Precision Floating-Point Values
	STARS_NN_vfmsub231sd,          // Fused Multiply-Subtract of Scalar Double-Precision Floating-Point Values
	STARS_NN_vfmsub231ss,          // Fused Multiply-Subtract of Scalar Single-Precision Floating-Point Values
	STARS_NN_vfmsubadd132pd,       // Fused Multiply-Alternating Subtract/Add of Packed Double-Precision Floating-Point Values
	STARS_NN_vfmsubadd132ps,       // Fused Multiply-Alternating Subtract/Add of Packed Single-Precision Floating-Point Values
	STARS_NN_vfmsubadd213pd,       // Fused Multiply-Alternating Subtract/Add of Packed Double-Precision Floating-Point Values
	STARS_NN_vfmsubadd213ps,       // Fused Multiply-Alternating Subtract/Add of Packed Single-Precision Floating-Point Values
	STARS_NN_vfmsubadd231pd,       // Fused Multiply-Alternating Subtract/Add of Packed Double-Precision Floating-Point Values
	STARS_NN_vfmsubadd231ps,       // Fused Multiply-Alternating Subtract/Add of Packed Single-Precision Floating-Point Values
	STARS_NN_vfnmadd132pd,         // Fused Negative Multiply-Add of Packed Double-Precision Floating-Point Values
	STARS_NN_vfnmadd132ps,         // Fused Negative Multiply-Add of Packed Single-Precision Floating-Point Values
	STARS_NN_vfnmadd132sd,         // Fused Negative Multiply-Add of Scalar Double-Precision Floating-Point Values
	STARS_NN_vfnmadd132ss,         // Fused Negative Multiply-Add of Scalar Single-Precision Floating-Point Values
	STARS_NN_vfnmadd213pd,         // Fused Negative Multiply-Add of Packed Double-Precision Floating-Point Values
	STARS_NN_vfnmadd213ps,         // Fused Negative Multiply-Add of Packed Single-Precision Floating-Point Values
	STARS_NN_vfnmadd213sd,         // Fused Negative Multiply-Add of Scalar Double-Precision Floating-Point Values
	STARS_NN_vfnmadd213ss,         // Fused Negative Multiply-Add of Scalar Single-Precision Floating-Point Values
	STARS_NN_vfnmadd231pd,         // Fused Negative Multiply-Add of Packed Double-Precision Floating-Point Values
	STARS_NN_vfnmadd231ps,         // Fused Negative Multiply-Add of Packed Single-Precision Floating-Point Values
	STARS_NN_vfnmadd231sd,         // Fused Negative Multiply-Add of Scalar Double-Precision Floating-Point Values
	STARS_NN_vfnmadd231ss,         // Fused Negative Multiply-Add of Scalar Single-Precision Floating-Point Values
	STARS_NN_vfnmsub132pd,         // Fused Negative Multiply-Subtract of Packed Double-Precision Floating-Point Values
	STARS_NN_vfnmsub132ps,         // Fused Negative Multiply-Subtract of Packed Single-Precision Floating-Point Values
	STARS_NN_vfnmsub132sd,         // Fused Negative Multiply-Subtract of Scalar Double-Precision Floating-Point Values
	STARS_NN_vfnmsub132ss,         // Fused Negative Multiply-Subtract of Scalar Single-Precision Floating-Point Values
	STARS_NN_vfnmsub213pd,         // Fused Negative Multiply-Subtract of Packed Double-Precision Floating-Point Values
	STARS_NN_vfnmsub213ps,         // Fused Negative Multiply-Subtract of Packed Single-Precision Floating-Point Values
	STARS_NN_vfnmsub213sd,         // Fused Negative Multiply-Subtract of Scalar Double-Precision Floating-Point Values
	STARS_NN_vfnmsub213ss,         // Fused Negative Multiply-Subtract of Scalar Single-Precision Floating-Point Values
	STARS_NN_vfnmsub231pd,         // Fused Negative Multiply-Subtract of Packed Double-Precision Floating-Point Values
	STARS_NN_vfnmsub231ps,         // Fused Negative Multiply-Subtract of Packed Single-Precision Floating-Point Values
	STARS_NN_vfnmsub231sd,         // Fused Negative Multiply-Subtract of Scalar Double-Precision Floating-Point Values
	STARS_NN_vfnmsub231ss,         // Fused Negative Multiply-Subtract of Scalar Single-Precision Floating-Point Values
	STARS_NN_vgatherdps,           // Gather Packed SP FP Values Using Signed Dword Indices
	STARS_NN_vgatherdpd,           // Gather Packed DP FP Values Using Signed Dword Indices
	STARS_NN_vgatherqps,           // Gather Packed SP FP Values Using Signed Qword Indices
	STARS_NN_vgatherqpd,           // Gather Packed DP FP Values Using Signed Qword Indices
	STARS_NN_vhaddpd,              // Add horizontally packed DP FP numbers
	STARS_NN_vhaddps,              // Add horizontally packed SP FP numbers
	STARS_NN_vhsubpd,              // Sub horizontally packed DP FP numbers
	STARS_NN_vhsubps,              // Sub horizontally packed SP FP numbers
	STARS_NN_vinsertf128,          // Insert Packed Floating-Point Values
	STARS_NN_vinserti128,          // Insert Packed Integer Values
	STARS_NN_vinsertps,            // Insert Packed Single Precision Floating-Point Value
	STARS_NN_vlddqu,               // Load Unaligned Packed Integer Values
	STARS_NN_vldmxcsr,             // Load Streaming SIMD Extensions Technology Control/Status Register
	STARS_NN_vmaskmovdqu,          // Store Selected Bytes of Double Quadword with NT Hint
	STARS_NN_vmaskmovpd,           // Conditionally Load Packed Double-Precision Floating-Point Values
	STARS_NN_vmaskmovps,           // Conditionally Load Packed Single-Precision Floating-Point Values
	STARS_NN_vmaxpd,               // Return Maximum Packed Double-Precision Floating-Point Values
	STARS_NN_vmaxps,               // Packed Single-FP Maximum
	STARS_NN_vmaxsd,               // Return Maximum Scalar Double-Precision Floating-Point Value
	STARS_NN_vmaxss,               // Scalar Single-FP Maximum
	STARS_NN_vminpd,               // Return Minimum Packed Double-Precision Floating-Point Values
	STARS_NN_vminps,               // Packed Single-FP Minimum
	STARS_NN_vminsd,               // Return Minimum Scalar Double-Precision Floating-Point Value
	STARS_NN_vminss,               // Scalar Single-FP Minimum
	STARS_NN_vmovapd,              // Move Aligned Packed Double-Precision Floating-Point Values
	STARS_NN_vmovaps,              // Move Aligned Four Packed Single-FP
	STARS_NN_vmovd,                // Move 32 bits
	STARS_NN_vmovddup,             // Move One Double-FP and Duplicate
	STARS_NN_vmovdqa,              // Move Aligned Double Quadword
	STARS_NN_vmovdqu,              // Move Unaligned Double Quadword
	STARS_NN_vmovhlps,             // Move High to Low Packed Single-FP
	STARS_NN_vmovhpd,              // Move High Packed Double-Precision Floating-Point Values
	STARS_NN_vmovhps,              // Move High Packed Single-FP
	STARS_NN_vmovlhps,             // Move Low to High Packed Single-FP
	STARS_NN_vmovlpd,              // Move Low Packed Double-Precision Floating-Point Values
	STARS_NN_vmovlps,              // Move Low Packed Single-FP
	STARS_NN_vmovmskpd,            // Extract Packed Double-Precision Floating-Point Sign Mask
	STARS_NN_vmovmskps,            // Move Mask to Register
	STARS_NN_vmovntdq,             // Store Double Quadword Using Non-Temporal Hint
	STARS_NN_vmovntdqa,            // Load Double Quadword Non-Temporal Aligned Hint
	STARS_NN_vmovntpd,             // Store Packed Double-Precision Floating-Point Values Using Non-Temporal Hint
	STARS_NN_vmovntps,             // Move Aligned Four Packed Single-FP Non Temporal
	STARS_NN_vmovntsd,             // Move Non-Temporal Scalar Double-Precision Floating-Point
	STARS_NN_vmovntss,             // Move Non-Temporal Scalar Single-Precision Floating-Point
	STARS_NN_vmovq,                // Move 64 bits
	STARS_NN_vmovsd,               // Move Scalar Double-Precision Floating-Point Values
	STARS_NN_vmovshdup,            // Move Packed Single-FP High and Duplicate
	STARS_NN_vmovsldup,            // Move Packed Single-FP Low and Duplicate
	STARS_NN_vmovss,               // Move Scalar Single-FP
	STARS_NN_vmovupd,              // Move Unaligned Packed Double-Precision Floating-Point Values
	STARS_NN_vmovups,              // Move Unaligned Four Packed Single-FP
	STARS_NN_vmpsadbw,             // Compute Multiple Packed Sums of Absolute Difference
	STARS_NN_vmulpd,               // Multiply Packed Double-Precision Floating-Point Values
	STARS_NN_vmulps,               // Packed Single-FP Multiply
	STARS_NN_vmulsd,               // Multiply Scalar Double-Precision Floating-Point Values
	STARS_NN_vmulss,               // Scalar Single-FP Multiply
	STARS_NN_vorpd,                // Bitwise Logical OR of Double-Precision Floating-Point Values
	STARS_NN_vorps,                // Bitwise Logical OR for Single-FP Data
	STARS_NN_vpabsb,               // Packed Absolute Value Byte
	STARS_NN_vpabsd,               // Packed Absolute Value Doubleword
	STARS_NN_vpabsw,               // Packed Absolute Value Word
	STARS_NN_vpackssdw,            // Pack with Signed Saturation (Dword->Word)
	STARS_NN_vpacksswb,            // Pack with Signed Saturation (Word->Byte)
	STARS_NN_vpackusdw,            // Pack with Unsigned Saturation
	STARS_NN_vpackuswb,            // Pack with Unsigned Saturation (Word->Byte)
	STARS_NN_vpaddb,               // Packed Add Byte
	STARS_NN_vpaddd,               // Packed Add Dword
	STARS_NN_vpaddq,               // Add Packed Quadword Integers
	STARS_NN_vpaddsb,              // Packed Add with Saturation (Byte)
	STARS_NN_vpaddsw,              // Packed Add with Saturation (Word)
	STARS_NN_vpaddusb,             // Packed Add Unsigned with Saturation (Byte)
	STARS_NN_vpaddusw,             // Packed Add Unsigned with Saturation (Word)
	STARS_NN_vpaddw,               // Packed Add Word
	STARS_NN_vpalignr,             // Packed Align Right
	STARS_NN_vpand,                // Bitwise Logical And
	STARS_NN_vpandn,               // Bitwise Logical And Not
	STARS_NN_vpavgb,               // Packed Average (Byte)
	STARS_NN_vpavgw,               // Packed Average (Word)
	STARS_NN_vpblendd,             // Blend Packed Dwords
	STARS_NN_vpblendvb,            // Variable Blend Packed Bytes
	STARS_NN_vpblendw,             // Blend Packed Words
	STARS_NN_vpbroadcastb,         // Broadcast a Byte Integer
	STARS_NN_vpbroadcastd,         // Broadcast a Dword Integer
	STARS_NN_vpbroadcastq,         // Broadcast a Qword Integer
	STARS_NN_vpbroadcastw,         // Broadcast a Word Integer
	STARS_NN_vpclmulqdq,           // Carry-Less Multiplication Quadword
	STARS_NN_vpcmpeqb,             // Packed Compare for Equal (Byte)
	STARS_NN_vpcmpeqd,             // Packed Compare for Equal (Dword)
	STARS_NN_vpcmpeqq,             // Compare Packed Qword Data for Equal
	STARS_NN_vpcmpeqw,             // Packed Compare for Equal (Word)
	STARS_NN_vpcmpestri,           // Packed Compare Explicit Length Strings, Return Index
	STARS_NN_vpcmpestrm,           // Packed Compare Explicit Length Strings, Return Mask
	STARS_NN_vpcmpgtb,             // Packed Compare for Greater Than (Byte)
	STARS_NN_vpcmpgtd,             // Packed Compare for Greater Than (Dword)
	STARS_NN_vpcmpgtq,             // Compare Packed Data for Greater Than
	STARS_NN_vpcmpgtw,             // Packed Compare for Greater Than (Word)
	STARS_NN_vpcmpistri,           // Packed Compare Implicit Length Strings, Return Index
	STARS_NN_vpcmpistrm,           // Packed Compare Implicit Length Strings, Return Mask
	STARS_NN_vperm2f128,           // Permute Floating-Point Values
	STARS_NN_vperm2i128,           // Permute Integer Values
	STARS_NN_vpermd,               // Full Doublewords Element Permutation
	STARS_NN_vpermilpd,            // Permute Double-Precision Floating-Point Values
	STARS_NN_vpermilps,            // Permute Single-Precision Floating-Point Values
	STARS_NN_vpermpd,              // Permute Double-Precision Floating-Point Elements
	STARS_NN_vpermps,              // Permute Single-Precision Floating-Point Elements
	STARS_NN_vpermq,               // Qwords Element Permutation
	STARS_NN_vpextrb,              // Extract Byte
	STARS_NN_vpextrd,              // Extract Dword
	STARS_NN_vpextrq,              // Extract Qword
	STARS_NN_vpextrw,              // Extract Word
	STARS_NN_vpgatherdd,           // Gather Packed Dword Values Using Signed Dword Indices
	STARS_NN_vpgatherdq,           // Gather Packed Qword Values Using Signed Dword Indices
	STARS_NN_vpgatherqd,           // Gather Packed Dword Values Using Signed Qword Indices
	STARS_NN_vpgatherqq,           // Gather Packed Qword Values Using Signed Qword Indices
	STARS_NN_vphaddd,              // Packed Horizontal Add Doubleword
	STARS_NN_vphaddsw,             // Packed Horizontal Add and Saturate
	STARS_NN_vphaddw,              // Packed Horizontal Add Word
	STARS_NN_vphminposuw,          // Packed Horizontal Word Minimum
	STARS_NN_vphsubd,              // Packed Horizontal Subtract Doubleword
	STARS_NN_vphsubsw,             // Packed Horizontal Subtract and Saturate
	STARS_NN_vphsubw,              // Packed Horizontal Subtract Word
	STARS_NN_vpinsrb,              // Insert Byte
	STARS_NN_vpinsrd,              // Insert Dword
	STARS_NN_vpinsrq,              // Insert Qword
	STARS_NN_vpinsrw,              // Insert Word
	STARS_NN_vpmaddubsw,           // Multiply and Add Packed Signed and Unsigned Bytes
	STARS_NN_vpmaddwd,             // Packed Multiply and Add
	STARS_NN_vpmaskmovd,           // Conditionally Store Dword Values Using Mask
	STARS_NN_vpmaskmovq,           // Conditionally Store Qword Values Using Mask
	STARS_NN_vpmaxsb,              // Maximum of Packed Signed Byte Integers
	STARS_NN_vpmaxsd,              // Maximum of Packed Signed Dword Integers
	STARS_NN_vpmaxsw,              // Packed Signed Integer Word Maximum
	STARS_NN_vpmaxub,              // Packed Unsigned Integer Byte Maximum
	STARS_NN_vpmaxud,              // Maximum of Packed Unsigned Dword Integers
	STARS_NN_vpmaxuw,              // Maximum of Packed Word Integers
	STARS_NN_vpminsb,              // Minimum of Packed Signed Byte Integers
	STARS_NN_vpminsd,              // Minimum of Packed Signed Dword Integers
	STARS_NN_vpminsw,              // Packed Signed Integer Word Minimum
	STARS_NN_vpminub,              // Packed Unsigned Integer Byte Minimum
	STARS_NN_vpminud,              // Minimum of Packed Unsigned Dword Integers
	STARS_NN_vpminuw,              // Minimum of Packed Word Integers
	STARS_NN_vpmovmskb,            // Move Byte Mask to Integer
	STARS_NN_vpmovsxbd,            // Packed Move with Sign Extend
	STARS_NN_vpmovsxbq,            // Packed Move with Sign Extend
	STARS_NN_vpmovsxbw,            // Packed Move with Sign Extend
	STARS_NN_vpmovsxdq,            // Packed Move with Sign Extend
	STARS_NN_vpmovsxwd,            // Packed Move with Sign Extend
	STARS_NN_vpmovsxwq,            // Packed Move with Sign Extend
	STARS_NN_vpmovzxbd,            // Packed Move with Zero Extend
	STARS_NN_vpmovzxbq,            // Packed Move with Zero Extend
	STARS_NN_vpmovzxbw,            // Packed Move with Zero Extend
	STARS_NN_vpmovzxdq,            // Packed Move with Zero Extend
	STARS_NN_vpmovzxwd,            // Packed Move with Zero Extend
	STARS_NN_vpmovzxwq,            // Packed Move with Zero Extend
	STARS_NN_vpmuldq,              // Multiply Packed Signed Dword Integers
	STARS_NN_vpmulhrsw,            // Packed Multiply High with Round and Scale
	STARS_NN_vpmulhuw,             // Packed Multiply High Unsigned
	STARS_NN_vpmulhw,              // Packed Multiply High
	STARS_NN_vpmulld,              // Multiply Packed Signed Dword Integers and Store Low Result
	STARS_NN_vpmullw,              // Packed Multiply Low
	STARS_NN_vpmuludq,             // Multiply Packed Unsigned Doubleword Integers
	STARS_NN_vpor,                 // Bitwise Logical Or
	STARS_NN_vpsadbw,              // Packed Sum of Absolute Differences
	STARS_NN_vpshufb,              // Packed Shuffle Bytes
	STARS_NN_vpshufd,              // Shuffle Packed Doublewords
	STARS_NN_vpshufhw,             // Shuffle Packed High Words
	STARS_NN_vpshuflw,             // Shuffle Packed Low Words
	STARS_NN_vpsignb,              // Packed SIGN Byte
	STARS_NN_vpsignd,              // Packed SIGN Doubleword
	STARS_NN_vpsignw,              // Packed SIGN Word
	STARS_NN_vpslld,               // Packed Shift Left Logical (Dword)
	STARS_NN_vpslldq,              // Shift Double Quadword Left Logical
	STARS_NN_vpsllq,               // Packed Shift Left Logical (Qword)
	STARS_NN_vpsllvd,              // Variable Bit Shift Left Logical (Dword)
	STARS_NN_vpsllvq,              // Variable Bit Shift Left Logical (Qword)
	STARS_NN_vpsllw,               // Packed Shift Left Logical (Word)
	STARS_NN_vpsrad,               // Packed Shift Right Arithmetic (Dword)
	STARS_NN_vpsravd,              // Variable Bit Shift Right Arithmetic
	STARS_NN_vpsraw,               // Packed Shift Right Arithmetic (Word)
	STARS_NN_vpsrld,               // Packed Shift Right Logical (Dword)
	STARS_NN_vpsrldq,              // Shift Double Quadword Right Logical (Qword)
	STARS_NN_vpsrlq,               // Packed Shift Right Logical (Qword)
	STARS_NN_vpsrlvd,              // Variable Bit Shift Right Logical (Dword)
	STARS_NN_vpsrlvq,              // Variable Bit Shift Right Logical (Qword)
	STARS_NN_vpsrlw,               // Packed Shift Right Logical (Word)
	STARS_NN_vpsubb,               // Packed Subtract Byte
	STARS_NN_vpsubd,               // Packed Subtract Dword
	STARS_NN_vpsubq,               // Subtract Packed Quadword Integers
	STARS_NN_vpsubsb,              // Packed Subtract with Saturation (Byte)
	STARS_NN_vpsubsw,              // Packed Subtract with Saturation (Word)
	STARS_NN_vpsubusb,             // Packed Subtract Unsigned with Saturation (Byte)
	STARS_NN_vpsubusw,             // Packed Subtract Unsigned with Saturation (Word)
	STARS_NN_vpsubw,               // Packed Subtract Word
	STARS_NN_vptest,               // Logical Compare
	STARS_NN_vpunpckhbw,           // Unpack High Packed Data (Byte->Word)
	STARS_NN_vpunpckhdq,           // Unpack High Packed Data (Dword->Qword)
	STARS_NN_vpunpckhqdq,          // Unpack High Packed Data (Qword->Xmmword)
	STARS_NN_vpunpckhwd,           // Unpack High Packed Data (Word->Dword)
	STARS_NN_vpunpcklbw,           // Unpack Low Packed Data (Byte->Word)
	STARS_NN_vpunpckldq,           // Unpack Low Packed Data (Dword->Qword)
	STARS_NN_vpunpcklqdq,          // Unpack Low Packed Data (Qword->Xmmword)
	STARS_NN_vpunpcklwd,           // Unpack Low Packed Data (Word->Dword)
	STARS_NN_vpxor,                // Bitwise Logical Exclusive Or
	STARS_NN_vrcpps,               // Packed Single-FP Reciprocal
	STARS_NN_vrcpss,               // Scalar Single-FP Reciprocal
	STARS_NN_vroundpd,             // Round Packed Double Precision Floating-Point Values
	STARS_NN_vroundps,             // Round Packed Single Precision Floating-Point Values
	STARS_NN_vroundsd,             // Round Scalar Double Precision Floating-Point Values
	STARS_NN_vroundss,             // Round Scalar Single Precision Floating-Point Values
	STARS_NN_vrsqrtps,             // Packed Single-FP Square Root Reciprocal
	STARS_NN_vrsqrtss,             // Scalar Single-FP Square Root Reciprocal
	STARS_NN_vshufpd,              // Shuffle Packed Double-Precision Floating-Point Values
	STARS_NN_vshufps,              // Shuffle Single-FP
	STARS_NN_vsqrtpd,              // Compute Square Roots of Packed Double-Precision Floating-Point Values
	STARS_NN_vsqrtps,              // Packed Single-FP Square Root
	STARS_NN_vsqrtsd,              // Compute Square Rootof Scalar Double-Precision Floating-Point Value
	STARS_NN_vsqrtss,              // Scalar Single-FP Square Root
	STARS_NN_vstmxcsr,             // Store Streaming SIMD Extensions Technology Control/Status Register
	STARS_NN_vsubpd,               // Subtract Packed Double-Precision Floating-Point Values
	STARS_NN_vsubps,               // Packed Single-FP Subtract
	STARS_NN_vsubsd,               // Subtract Scalar Double-Precision Floating-Point Values
	STARS_NN_vsubss,               // Scalar Single-FP Subtract
	STARS_NN_vtestpd,              // Packed Double-Precision Floating-Point Bit Test
	STARS_NN_vtestps,              // Packed Single-Precision Floating-Point Bit Test
	STARS_NN_vucomisd,             // Unordered Compare Scalar Ordered Double-Precision Floating-Point Values and Set EFLAGS
	STARS_NN_vucomiss,             // Scalar Unordered Single-FP Compare and Set EFLAGS
	STARS_NN_vunpckhpd,            // Unpack and Interleave High Packed Double-Precision Floating-Point Values
	STARS_NN_vunpckhps,            // Unpack High Packed Single-FP Data
	STARS_NN_vunpcklpd,            // Unpack and Interleave Low Packed Double-Precision Floating-Point Values
	STARS_NN_vunpcklps,            // Unpack Low Packed Single-FP Data
	STARS_NN_vxorpd,               // Bitwise Logical OR of Double-Precision Floating-Point Values
	STARS_NN_vxorps,               // Bitwise Logical XOR for Single-FP Data
	STARS_NN_vzeroall,             // Zero All YMM Registers
	STARS_NN_vzeroupper,           // Zero Upper Bits of YMM Registers

	// Transactional Synchronization Extensions

	STARS_NN_xabort,               // Transaction Abort
	STARS_NN_xbegin,               // Transaction Begin
	STARS_NN_xend,                 // Transaction End
	STARS_NN_xtest,                // Test If In Transactional Execution

	// Virtual PC synthetic instructions

	STARS_NN_vmgetinfo,            // Virtual PC - Get VM Information
	STARS_NN_vmsetinfo,            // Virtual PC - Set VM Information
	STARS_NN_vmdxdsbl,             // Virtual PC - Disable Direct Execution
	STARS_NN_vmdxenbl,             // Virtual PC - Enable Direct Execution
	STARS_NN_vmcpuid,              // Virtual PC - Virtualized CPU Information
	STARS_NN_vmhlt,                // Virtual PC - Halt
	STARS_NN_vmsplaf,              // Virtual PC - Spin Lock Acquisition Failed
	STARS_NN_vmpushfd,             // Virtual PC - Push virtualized flags register
	STARS_NN_vmpopfd,              // Virtual PC - Pop virtualized flags register
	STARS_NN_vmcli,                // Virtual PC - Clear Interrupt Flag
	STARS_NN_vmsti,                // Virtual PC - Set Interrupt Flag
	STARS_NN_vmiretd,              // Virtual PC - Return From Interrupt
	STARS_NN_vmsgdt,               // Virtual PC - Store Global Descriptor Table
	STARS_NN_vmsidt,               // Virtual PC - Store Interrupt Descriptor Table
	STARS_NN_vmsldt,               // Virtual PC - Store Local Descriptor Table
	STARS_NN_vmstr,                // Virtual PC - Store Task Register
	STARS_NN_vmsdte,               // Virtual PC - Store to Descriptor Table Entry
	STARS_NN_vpcext,               // Virtual PC - ISA extension

	STARS_NN_last,

};

// Aliases to match capitalization in beaEngine.
#define STARS_NN_prefetchNTA STARS_NN_prefetchnta

#endif
