#ifndef STARSOp_h
#define STARSOp_h

#include <stdint.h>
#include <assert.h>
#include <map>

#include "interfaces/STARSTypes.h"

class STARS_op_t
{
	public:
		
		// Constructors
		STARS_op_t() = default;

		// Clone method (deep copy).
		virtual STARSOpndTypePtr clone(void) const = 0;

		// Operators
		virtual bool operator<(const STARS_op_t &rOp) const = 0; // Less-than operator for use in STL ordered containers, e.g. sets.

		// Get methods
		virtual unsigned int GetOpGlobalIndex(void) const = 0; // Get STARS SSA name index.
		virtual STARS_ea_t GetAddr(void) const = 0;
		virtual unsigned char GetOpType(void) const = 0; // Get type o_reg, o_displ, etc.
		virtual STARS_regnum_t GetReg(void) const = 0;  // Get reg field of operand, whether it is an addressing reg or directly used reg in register operand
		virtual STARS_regnum_t GetSegReg(void) const = 0; // Get segment register of operand; R_none if no segment register
		virtual char GetSIB(void) const = 0; // Get x86 SIB byte (dense encoding of base reg, index reg, and scale factor)
		virtual int MDGetSIBBaseReg(void) const = 0; // Get x86 SIB byte base register
		virtual short MDGetSIBIndexReg(void) const = 0; // Get x86 SIB byte index register
		virtual int GetSIBScaleFactor(void) const = 0; // Extract scale factor from SIB byte
		virtual char GetSpecFlag4(void) const { return 0; } // ida debugging.
		virtual STARS_uval_t GetImmedValue(void) const = 0; // Get value field for immediate operands; uint32 for x86-32, uint64 for x86-64
		virtual char GetOpDtyp(void) const = 0;  // Get field that determines byte width
		virtual uint16_t GetByteWidth(void) const = 0;
		virtual void MDExtractAddressFields(int &BaseReg, int &IndexReg, uint16_t &Scale, STARS_ea_t &Offset);

		// Set methods
		virtual void SetByteWidth(uint16_t ByteWidth) = 0; // Change default byte width of opnd
		virtual void DoubleRegWidth(void) = 0; // Double the width of opnd if it is a subword register
		virtual void SetOpGlobalIndex(std::size_t index) = 0; // Set STARS SSA name index.
		virtual void SetReg(STARS_regnum_t NewReg) = 0;
		virtual void SetAddr(STARS_ea_t NewAddr) = 0;
		virtual void SetSIB(char value) = 0;
		virtual void SetNotVisible(void) = 0; // Set operand to type that is implicit, not explicit in disasm
		virtual void SetTypeToMemDisplacement(void) = 0; // change type during stack normalization
		virtual void SetTypeToMemNoDisplacement(void) = 0; // change type during stack normalization
		virtual void ClearSIB(void) = 0; // Set memory operand to have no SIB byte.
		virtual void CleanOpndEncoding(void); // Get rid of unnecessary SIB byte encodings, etc.

		// Query methods
		virtual bool IsVisible(void) const = 0;  // Operand appears in disasm, vs. being implicit, e.g. "imul ecx" also has eax opnd
		virtual bool IsRegOp(void) const = 0;
		virtual bool IsImmedOp(void) const = 0;
		virtual bool IsVoidOp(void) const = 0;
		virtual bool IsMemDisplacementOp(void) const = 0;
		virtual bool IsStaticMemOp(void) const = 0;
		virtual bool IsMemNoDisplacementOp(void) const = 0;
		virtual bool IsMemOp(void) const = 0;  // any of the previous three types
		virtual bool HasSIBByte(void) const = 0;
		virtual bool IsFloatingPointRegOp(void) const = 0;
		virtual bool IsMMXRegOp(void) const = 0;
		virtual bool IsXMMRegOp(void) const = 0;
		virtual bool IsYMMRegOp(void) const = 0;
		virtual bool IsTestRegOp(void) const = 0;
		virtual bool IsDebugRegOp(void) const = 0;
		virtual bool IsControlRegOp(void) const = 0;
		virtual bool MDIsKnownOpType(void) const = 0;
		virtual bool MDIsSpecialRegOpType(void) const = 0;  // special debug reg, control or task reg, MMX/XMM/YMM registers
		virtual bool MatchesReg(STARS_regnum_t RegNum) const = 0;
		virtual bool IsNearPointer(void) const = 0;
		virtual bool IsFarPointer(void) const = 0;
		virtual bool HasSegReg(void) const = 0; // Has a segment register

		// Printing methods.
		virtual void Dump() { }


};

#endif



