#ifndef STARS_IRDB_Op_h
#define STARS_IRDB_Op_h

#include <stdint.h>
#include <assert.h>
#include "interfaces/STARSTypes.h"
#include <libIRDB-core.hpp>
#include "interfaces/abstract/STARSOp.h"
#include <string.h>

// forward decls
class STARS_IRDB_op_t;

// to help easy porting if this ever changes.
typedef std::shared_ptr<STARS_IRDB_op_t> STARSIRDBOpndTypePtr;

class STARS_IRDB_op_t : public STARS_op_t
{
	public:
		
		// Constructors
		STARS_IRDB_op_t() { Init(); }
		STARS_IRDB_op_t(const DISASM &d, int indx, const ARGTYPE &the_arg, const int length=0);
	


		// Clone method (deep copy).
		virtual STARSOpndTypePtr clone(void) const 
		{ 
			std::shared_ptr<STARS_IRDB_op_t> p=std::make_shared<STARS_IRDB_op_t>();
			p->global_index = global_index;
			p->SegReg = SegReg;
			p->byteWidth = byteWidth;
			p->OpType = OpType;
			(void) memcpy(&(p->operand), &operand, sizeof(p->operand));
			p->visible = visible;
			return p;
		}

		// Operators
		virtual bool operator<(const STARS_op_t &rOp) const;  // Less-than operator for use in STL ordered containers, e.g. sets.

		// Get methods

// maybe convert Set/GetOpGlobalIndex to take/return a typedef.
//typedef unsigned int SSANameIndex_t

		// Get/SetOpGlobalIndex just needs to record a unsigned int so clone can copy it.
		virtual unsigned int GetOpGlobalIndex(void) const { return global_index; } // Get STARS SSA name index.

		// e.g. mov [eax+addr], 0 		==> return addr, assert
		// e.g. mov [eax+ebx+8+addr], 0 	==> return addr, assert
		// e.g. jeq +5 				==> 0x8048005 
		// e.g. jmp +5 				==> 0x8048005 
		// e.g. mov eax <- 0x8048f00 		==> assert, assert
		// e.g. call +5 			==> 0x8048005
		// e.g. mov [0x8048f00] <- eax 		==> 0x8048f00, assert
		// e.g. mov fs:[0x8048f00], eax 	==> 0x8048f00, assert
		// e.g. lea eax <- [0x8048f00] 		==> assert, 0x8048f00
		// e.g. mov eax <- 0x8048f00 		==> assert, assert
		// e.g. inc eax 			==> assert
		virtual STARS_ea_t GetAddr(void) const 
		{
			switch(OpType)
			{
				case op_Mem: return operand.mem.disp;
				case op_Imm: return operand.imm.imm;
				case op_Addr: return operand.addr.addr;
				default: return STARS_BADADDR;
			}
		}

		// should be straightforward enough.
		// jmp +5 ==> return o_near
		// mov [ebx+ecx*4+40],0  ==> return STARS_o_displ, STARS_o_imm
		// mov [ebx+ecx*4],rax  ==> return STARS_o_phrase, STARS_o_reg
		// mov [ebx], reg  ==> return STARS_o_phrase, STARS_o_reg
		// mov [ebx+40],imm  ==> return STARS_o_displ, STARS_o_imm
		// mov [0x8048f040], 0  ==> return STARS_o_mem, STARS_o_imm
		// mov mm0, [mem]	==> return STARS_o_mmxreg, STARS_o_phrase
		// mov xmm0, [mem]	==> return STARS_o_xmmreg, STARS_o_phrase
		// mov ymm0, [mem]	==> return o_ymmreg, o_phrase
		// also see intel.hpp:56 for o_trreg, o_crreg, o_dbreg, o_fpreg.
		virtual unsigned char GetOpType(void) const { return OpType; } // Get type o_reg, o_displ, etc.

		
		// STARS_x86_R_al
		// STARS_x86_R_ah
		// STARS_x86_R_ax	(means ax, eax, rax)
		// STARS_x86_R_mm0	
		// STARS_x86_R_xmm0	
		// STARS_x86_R_ymm0	
		// STARS_x86_R_st0 	(fp stack reg 0)
		// probably also CR, TR, and DB regs.  
		// seg registers aren't 
		// see STARSTypes.h: enum STARS_RegNo
		virtual STARS_regnum_t GetReg(void) const 
		{ 
			switch(OpType)
			{
				case op_Reg:
					return operand.reg.RegNum;
				case op_Mem:
					return operand.mem.base;
				default:
					// immediates, etc. don't have a reg.
					return STARS_x86_R_none;
			}
		}  // Get reg field of operand, whether it is an addressing reg or directly used reg in register operand

		// mov fs:[0x80481234], eax ==> return seg_fs, assert.
		virtual STARS_regnum_t GetSegReg(void) const { return SegReg;  } // Get segment register of operand; R_none if no segment register

		// return SIB byte if exists.
		 // Get x86 SIB byte (dense encoding of base reg, index reg, and scale factor)
		virtual char GetSIB(void) const;

		virtual int MDGetSIBBaseReg(void) const { return operand.mem.base;  } // Get x86 SIB byte base register
		virtual short MDGetSIBIndexReg(void) const { return operand.mem.index; } // Get x86 SIB byte index register
		virtual int GetSIBScaleFactor(void) const { return operand.mem.scale; } // Extract scale factor from SIB byte

		// Get value field for immediate operands; uint32 for x86-32, uint64 for x86-64
		virtual STARS_uval_t GetImmedValue(void) const { assert(OpType==op_Imm);  return operand.imm.imm; } 

		// return STARS_dt_byte, etc.
		// see STARSTypes.h:112
		// probably don't need it.
		virtual char GetOpDtyp(void) const { return OpType; }  // Get field that determines byte width

		// return 1, 2, 4, 8, 16, etc.
		virtual uint16_t GetByteWidth(void) const;

		// Set methods
		virtual void SetByteWidth(uint16_t ByteWidth) { assert(OpType!=op_Void); byteWidth=ByteWidth; } // Change default byte width of opnd
		virtual void DoubleRegWidth(void) { assert(OpType!=op_Void); byteWidth*=2; } // Double the width of opnd if it is a subword register

		// where's this come from?
		virtual void SetOpGlobalIndex(std::size_t index) { global_index=index; } // Set STARS SSA name index.


		virtual void SetReg(STARS_regnum_t NewReg) 
		{ 
			switch(OpType)
			{
				case op_Reg:
					operand.reg.RegNum=(STARS_RegNo)NewReg;
				case op_Mem:
					operand.mem.base=(STARS_RegNo)NewReg;
			}
			return;
		}	
		virtual void SetAddr(STARS_ea_t NewAddr) { assert(OpType==op_Mem); operand.mem.disp=NewAddr; }
		virtual void SetSIB(char value);
		virtual void ClearSIB() 
		{ 
			assert(OpType==op_Mem); 
			operand.mem.scale=0; 
			operand.mem.index=STARS_x86_R_none; 
			operand.mem.hasSIB=false;

			/* note: we leave the base register and displacement alone */
		}

		
		virtual void SetNotVisible(void) { assert(OpType!=op_Void); visible=0; } // Set operand to type that is implicit, not explicit in disasm

		// this is used to change the type of a memory operand during stack normalization
		virtual void SetTypeToMemDisplacement(void) { assert(IsMemDisplacementOp()); } // SetAddr(non-zero) should already be called, which implicitly does this. 

		// this is used to change the type of a memory operand during stack normalization
		virtual void SetTypeToMemNoDisplacement(void) { assert(OpType==op_Mem && operand.mem.base==0); } // SetAddr(0) should already have been called 

		// Query methods
		virtual bool IsVisible(void) const { return visible && OpType!=op_Void; }
		virtual bool IsRegOp(void) const { return OpType==op_Reg; }
		virtual bool IsImmedOp(void) const { return OpType==op_Imm; }
		virtual bool IsVoidOp(void) const { return OpType==op_Void; }
		virtual bool IsMemDisplacementOp(void) const { return OpType==op_Mem && operand.mem.disp!=0; }
		virtual bool IsStaticMemOp(void) const { return OpType==op_Mem && operand.mem.base==STARS_x86_R_none && operand.mem.index==STARS_x86_R_none; }
		virtual bool IsMemNoDisplacementOp(void) const { return OpType==op_Mem && operand.mem.disp==0; }
		virtual bool IsMemOp(void) const { return OpType==op_Mem; } 
		virtual bool HasSIBByte(void) const { assert(OpType==op_Mem); return operand.mem.hasSIB; }
		virtual bool IsFloatingPointRegOp(void) const 
		{ if(OpType!=op_Reg) return false;   return STARS_x86_R_st0<= operand.reg.RegNum && operand.reg.RegNum<=STARS_x86_R_st7; }
		virtual bool IsMMXRegOp(void) const 
		{ if(OpType!=op_Reg) return false;  return STARS_x86_R_mm0<= operand.reg.RegNum && operand.reg.RegNum<=STARS_x86_R_mm7; }
		virtual bool IsXMMRegOp(void) const 
		{ if(OpType!=op_Reg) return false;  return STARS_x86_R_xmm0<= operand.reg.RegNum && operand.reg.RegNum<=STARS_x86_R_xmm15; }
		virtual bool IsYMMRegOp(void) const 
		{ if(OpType!=op_Reg) return false;  return STARS_x86_R_ymm0<= operand.reg.RegNum && operand.reg.RegNum<=STARS_x86_R_ymm15; }
		virtual bool IsTestRegOp(void) const { return false; }
		virtual bool IsDebugRegOp(void) const { return false; }
		virtual bool IsControlRegOp(void) const { return false; }
		virtual bool MDIsKnownOpType(void) const { return OpType!=op_Void; }
		virtual bool MDIsSpecialRegOpType(void) const { return false; }  // special debug reg, control or task reg, MMX/XMM/YMM registers
		virtual bool MatchesReg(STARS_regnum_t RegNum) const 
		{ 
			return OpType==op_Reg && RegNum == operand.reg.RegNum; 
		}
		virtual bool IsNearPointer(void) const { return OpType==op_Addr; }
		virtual bool IsFarPointer(void) const { return false; }
		virtual bool HasSegReg(void) const { return (SegReg != STARS_x86_R_none); } // Has a segment register


// conversion of op to new type
		virtual void MakeVoidOpnd(void)  { Init(); }
		virtual void MakeImmediateOpnd(STARS_uval_t value)  { Init(); OpType=op_Imm; operand.imm.imm=value; }
		virtual void MakeRegOpnd(STARS_regnum_t RegNum)  
		{ 
			Init(); 
			OpType=op_Reg; 
			operand.reg.RegNum=(STARS_RegNo)RegNum; 
		}
		virtual void MakeFloatingPointRegOpnd(STARS_regnum_t RegNum)  { Init(); OpType=op_FPReg; operand.reg.RegNum=(STARS_RegNo)RegNum; }
		virtual void MakeMMXRegOpnd(STARS_regnum_t RegNum)  { Init(); OpType=op_MMXReg; operand.reg.RegNum=(STARS_RegNo)RegNum; }
		virtual void MakeXMMRegOpnd(STARS_regnum_t RegNum)  { Init(); OpType=op_XMMReg; operand.reg.RegNum=(STARS_RegNo)RegNum; }
		virtual void MakeYMMRegOpnd(STARS_regnum_t RegNum)  { Init(); OpType=op_YMMReg; operand.reg.RegNum=(STARS_RegNo)RegNum; }
		virtual void MakeMemDisplacementOpnd(STARS_regnum_t BaseRegNum, 
			STARS_regnum_t IndexRegNum, uint16_t ScaleFactor, STARS_ea_t offset)  		
		{
			Init();
			OpType=op_Mem;
			operand.mem.base=(STARS_RegNo)BaseRegNum;
			operand.mem.index=(STARS_RegNo)IndexRegNum;
			operand.mem.scale=(STARS_RegNo)ScaleFactor;
			operand.mem.disp=offset;
		}
		virtual void MakeMemPhraseOpnd(STARS_regnum_t BaseRegNum, STARS_regnum_t IndexRegNum, uint16_t ScaleFactor)  	
		{ return MakeMemDisplacementOpnd(BaseRegNum,IndexRegNum,ScaleFactor,0); }

		virtual void MakeNearPointerOpnd(STARS_ea_t TargetAddr) 
		{
			Init();
			OpType=op_Addr;
			operand.addr.addr=TargetAddr;
		}



	private:
		// what's the global_index for Get/SetOpGlobalIndex
		std::size_t  global_index;

		STARS_RegNo SegReg;
		int byteWidth;

		enum OperandType_t { op_Void, op_Reg, op_Mem, op_Imm, op_CrReg, op_DrReg, op_Eflags, op_MMXReg, op_XMMReg, op_YMMReg, op_FPReg, op_Mxcsr, op_SegReg, op_Addr } OpType;
		union 
		{
			struct 
			{
				STARS_RegNo RegNum;
			} reg;
			struct 
			{
				STARS_RegNo base;
				STARS_RegNo index;
				unsigned int scale:4;	 // can take values 0, 1, 2, 4, 8.
				libIRDB::virtual_offset_t disp;	// may be 64-bit imm in some x86-64 instructions.
				bool hasSIB:1;
			} mem;
			struct 
			{
				libIRDB::virtual_offset_t imm;	// may be 64-bit imm in some x86-64 instructions.
				bool pc_rel;			// may be pcrel immediate in some cases.
			} imm;
			struct
			{
				 STARS_ea_t addr;
			} addr;
	
		
		} operand;
		bool visible;


		void Init() 
		{ 
			SegReg=STARS_x86_R_none;
			OpType=op_Void;

			// init the operand to 0, because the it should have no semantically valid meaning in any sense anyhow if OpType==Void.
			memset(&operand,0,sizeof(operand));

			visible=0;
			byteWidth=0;
		}

		virtual void Dump();

			

};

#endif



